function Asufit(action)

%------------------------------------------------------------------------------
% FILENAME: ASUFIT.M
%
% This is ASUFIT, a general fitting routine written under MATLAB for fitting
% transient emission and absorbance change data.  Any nonprofit organization
% is welcome to use this without charge, but we ask that the program authors
% be acknowleged. Program authors are:
%
%     Evaldas Katilius
%	  Joel Hindorff
%     Neal Woodbury
%
% This file creates the main figure window that is the ASUFIT graphical user
% interface.  The commands in this file set up the uicontrols and prepare
% the MATLAB workspace.
%
% Possible actions of this function are:
%     'initialize'
%     'spc'
%     'abs'
%     'svd'
%     'off'
%------------------------------------------------------------------------------

% Figure and axes handles
global mainWindow
global spcDataGrid
global spcResidGrid
global absDataGrid1
global absDataGrid2
global absResidGrid1
global absResidGrid2
global svdGrid1
global svdGrid2
global svdGrid3
global absIntSetAxes1
global absIntSetAxes2
global spcIntSetAxes

% Menu handles
global expFile
global kineticsFile
global spectrumFile
global ampSpecFile
global spcFitSave
global plot3D
global ampSpecPlot
global subtract
global revert
global spcGlobal
global perform
global components
global glob
global spec
global absDispersion
global restore
global average
global optionsMenu
global plotMenu
     
% Common uicontrol handles
global textBox
global optionsbutton
global plotbutton
global svdbutton
global action1button
global action2button
global action3button

% SPC uicontrol handles
global spcSpectrumText
global spcPromptText
global spcDateText
global spcSpectrumBox
global spcPromptBox
global spcDateBox
global spcExciteText
global spcEmitText
global spctpchText
global spcExciteBox
global spcEmitBox
global spctpchBox
global spcShiftNumText
global spcErrorText
global spcGlobalErrorText
global spcShiftNumBox
global spcErrorBox
global spcGlobalErrorBox
global spcTaoAnsText
global spcTaoAnsBox
global spcAmpAnsText
global spcAmpAnsBox
global spcIntPlot
global spcLeftEnd
global spcRightEnd
global spcTog
global spcPlot
global spcInd
global spcPlotMode
global spcResidPlot

% Absorbance uicontrol handles

global absCurrentWlText
global absCurrentWlBox
global absCurrentTimeText
global absCurrentTimeBox
global absAbsorbText
global absAbsorbBox
global absTaoAnsText
global absTaoAnsBox
global absErrorText
global absErrorBox

% Options screen handles
global spcNumTaoText
global spcNumTaoBox
global spcTaoText
global spcTaoBox
global spcNumTaoFixedText
global spcNumTaoFixedBox
global spcTaoFixedText
global spcTaoFixedBox
global spcNumAllTaoText
global spcNumAllTaoBox
global spcAllTaoText
global spcAllTaoBox
global spcshifttext
global spcshift
%global spcFTolText
%global spcFTolBox
global spcMaxItText
global spcMaxItBox

global absNumTaoText
global absNumTaoBox
global absTaoText
global absTaoBox
global absNumTaoFixedText
global absNumTaoFixedBox
global absTaoFixedText
global absTaoFixedBox
global absNumAllTaoText
global absNumAllTaoBox
global absAllTaoText
global absAllTaoBox
global absPulseWidthText
global absPulseWidthBox
global absShiftGuessText
global absShiftGuessBox
global abscheckboxtext
global abscheckbox1
global abscheckbox2
global abscheckbox3
global absMaxItText
global absMaxItBox
global absDisperCoeffsText
global absDisperCoeffsBox
global absDisperText
global absDisper1Hypertext
global absDisper2Hypertext
global absDisper3Hypertext
global absIntPlot1
global absIntPlot2
global absLeftEnd1
global absRightEnd1
global absLeftEnd2
global absRightEnd2
global absTog1
global absTog2
global absPlotKinSpec
global absInd1
global absInd2
global absPlot1
global absPlot2
global absResidPlot1
global absResidPlot2
global abstimeaxis

%SVD
global svdNumVecsText
global svdNumVecsBox
global svdTimeText
global svdTimeBox
global svdWavelengthText
global svdWavelengthBox
global svdErrorText
global svdErrorBox
global svdWeightText
global svdWeightBox
global svdSurfOld
global svdSurfNew
global svdGlobalTogHypertext
global svdGlobalToggle

%Amps spec
global ampPlotAll
global ampInd
global ampSpecWindow
global ampGrid
global ampUpbutton
global ampDownbutton
global ampClosebutton
global ampTextBox
global ampSwitchbutton

% Absorbance 3D plot variables
global abs3dFig
global abs3dAxes
global abs3dView
global horizRotBox
global vertRotBox

% Book keeping variables

global analysisType %  == 1 -> SPC/IRF
                    %  == 2 -> Absorbance
                    %  == 3 -> SVD of Absorbance
                    % otherwise, no data present in workspace
global dispersionStatus % == 0 -> no dispersion correction
                        % == 1 -> correct during fit
                        % == 2 -> data already changed
global dataIntervalStatus % == 0 -> use all of the data
                          % == 1 -> use interval for fit
                          % == 2 -> use interval now
global backupTypes % == 0 -> no backups
                   % == 1 -> dataBackup1 is data before dispersion and SVD
                   %         dataBackup2 is data with dispersion but before SVD
                   % == 2 -> dataBackup1 is data before dispersion and SVD
                   %         dataBackup2 is data with SVD but before dispersion
global svdAnal
global abstimelin
global fitmodel
                   

% Real variables
global data
global dataMax
global dataMin
global fit
global irf
global amps
global weight
global emit
global excite
global tpch
global date
global spcFilenames
global irfFilenames
global numTao
global tao
global numTaoFixed
global taoFixed
global pulseWidth
global absFilename
global restoreName
global absYTicks
global wavelength
global t
global dataFit
global disperCoeffs
global dispvecguess
global leftInt
global rightInt
global xTol
global fTol
global maxIt
global OPTIONS
global OPT_STOP
global paramsAns
global chisq
global taoAns
global shiftGuess
global shift
global tzero
global fitError
global dataBackup
global U
global S
global V
global numVecs
global dispersionVector
global pathname
global numTimePoints1
global numTimePoints2
global fitconst
global name


if (nargin < 1)
    action = 'initialize';
end

if (strcmp(action, 'initialize'))
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets up Asufit for the first time, no data is present in
% the workspace.
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% --- The following commands set up the figure window and all plot axes

	 pathname = 'c:\'; %default data pathname
    mainWindow = ... % Main figure window
        figure(...
            'Name',        'ASUFIT-Data Analysis Software',...
            'NumberTitle', 'off',...
            'MenuBar', 'none',...
            'Units',       'normalized',...
            'Position',    [.05 .05 .9 .9],...
            'Color',       [0.8 0.8 0.8],...
            'Visible',     'off'...
        );
    spcDataGrid = ... % SPC data axes
        axes(...
            'Position',  [0.06 .3 0.88 0.55],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    spcResidGrid = ... % SPC fit residual axes
        axes(...
            'Position', [0.06 .2 0.88 0.07],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    absDataGrid1 = ... % Temporal axes for absorbance
        axes(...
            'Position', [0.05 .37 0.425 0.47],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    absDataGrid2 = ... % Spectral axes for absorbance
        axes(...
            'Position', [0.525 .37 0.425 0.47],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    absResidGrid1 = ... % Temporal residual axes
        axes( ...
            'Position', [0.05 .12 0.425 0.2],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    absResidGrid2 = ... % Spectral residual axes
        axes( ...
            'Position', [0.525 .12 0.425 0.2],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    absIntSetAxes1 = ...
        axes(...
            'Position', [.07 .45 0.4 0.42],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
    );
   absIntSetAxes2 = ...
        axes(...
            'Position', [.53 .45 0.4 0.42],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
    );
    svdGrid1 = ... % Temporal residual axes
        axes( ...
            'Position', [0.05 .47 0.35 0.37],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
        );
    svdGrid2 = ... % Spectral residual axes
        axes( ...
            'Position', [0.45 .47 0.35 0.37],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
            );
         svdGrid3 = ... %component axes
            axes( ...
            'Position', [0.45 .05 0.35 0.37],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
            );
    spcIntSetAxes = ...
        axes(...
            'Position', [.07 .45 0.86 0.42],...
            'Visible',  'off',...
            'DrawMode', 'fast'...
    );
    
% --- The following commands set up the menus on the menu bar

    dataFile = ... % Pull-down menu for file operations
        uimenu(mainWindow,...
            'Label', '&File'...
        );
    open = ... % Sub-menu to open data files
        uimenu(dataFile,...
            'Label', '&Open'...
        );
    % Option for Stanford ASCII format
       % uimenu(open,...
        %    'Label',    '&Stanford ASCII',...
         %   'CallBack', 'Asufit(''openStanfAsc'')'...
        %);
    % Option for EXP format
        uimenu(open,...
            'Label',    '&Exp',...
            'CallBack', 'Asufit(''openExpFile'')'...
        );
    % Option for SPC/IRF format
        uimenu(open,...
            'Label',    '&SPC/IRF files',...
            'CallBack', 'Asufit(''openSpc'')'...
        );
    saveas = ... % Sub-menu to save data to files
        uimenu(dataFile,...
            'Label', '&Save'...
            );    
    expFile = ... %option for saving exp file
        uimenu(saveas,...
            'Label',    '&Exp',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''saveExpFile'')'...
        );
    kineticsFile = ... % Option for saving on-screen kinetics to file
        uimenu(saveas,...
            'Label',    'On-screen &kinetics',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''saveAbsKinetic'');'...
        );
    spectrumFile = ... % Option for saving on-screen spectrum
        uimenu(saveas,...
            'Label',    'On-screen &spectrum',...
             'Enable',   'off',...
            'CallBack', 'Asufit(''saveAbsSpectrum'');'...
        );
    ampSpecFile = ... % Option for saving the amplitudes
        uimenu(saveas,...
            'Label',    '&Amplitudes',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''saveAbsAmpSpec'');'...
        );
    spcFitSave = ... % Option for saving SPC fit and plot
        uimenu(saveas,...
            'Label',    '&SPC fit and plot',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''spcFitSave'');'...
        );
    restore = ... % Option for restoring absorbance data
        uimenu(dataFile,...
            'Label',    '&Restore data',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''restoreData'');'...
        );
     average= ... %option for averaging multiple data sets
        uimenu(dataFile,...
        'Label', '&Average EXP data',...
        'Enable', 'on',...
        'CallBack', 'Asufit(''averageData'');');
    plotMenu = ... % Pull-down menu for plot operations
        uimenu(mainWindow,...
            'Label','&Plot'...
        );
    % Plot the current data
        uimenu(plotMenu,...
            'Label',    '&Plot data',...
            'CallBack', 'Asufit(''plot'')'...
        );
    plot3D = ... % Surface plot of data
        uimenu(plotMenu,...
            'Label',    '&Surface plot',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''abs3dPlot'');'...
        );
    ampSpecPlot = ... % Plot amplitude spectra
        uimenu(plotMenu,...
            'Label',    '&Amplitude spectra',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''ampSpecPlot'');'...
        );
    spcGlobal = ... % Show global results of SPC fit
        uimenu(plotMenu,...
            'Label','SPC Global results window',...
            'Enable','off',...
            'CallBack','Asufit(''spcGlobalResultWindow'');'...
         );    
    
     optionsMenu = ... % Pull-down menu of other options
        uimenu(mainWindow,...
            'Label', '&Options'....
        );
    % Option to lock rate constants to input values
      abstimeaxis =  uimenu(optionsMenu,...
            'Label',    '&Change time axis',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''abstimechange'');'...
        );
    absDispersion = ... % Launch dispersion correction figure window
        uimenu(optionsMenu,...
            'Label',    'Apply dispersion correction',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''dispersionNow'');'...
        );
    subtract = ... % Subtract the baseline from the data
        uimenu(optionsMenu,...
            'Label',    'Subtract baseline',...
            'Enable',   'off',...
            'CallBack', 'Asufit(''subtract'')'...
        );
    %Zoom menu
      z= uimenu(mainWindow,...
            'Label',    'Zoom');
         uimenu(z,...
            'Label',    'Zoom on',...
            'CallBack', 'Asufit(''zoomon'')'...
            );    
         uimenu(z,...
            'Label',    'Zoom off',...
            'CallBack', 'Asufit(''zoomoff'')'...
            );    
   % Grid line menu     
   w= uimenu (mainWindow, 'label', 'Grid lines');
   
        uimenu(w,...
            'Label',    'Grid lines on',...
            'CallBack', 'asufit(''gridson'')');
     uimenu(w,...
            'Label',    'Grid lines off',...
            'CallBack', 'asufit(''gridsoff'')');

         % Print command
        uimenu(mainWindow,...
            'Label',    'Print',...
            'CallBack', 'Asufit(''print'')'...
            );    
         
         % Quit command 
         uimenu(mainWindow,...
            'Label',    '&Quit',...
            'CallBack', 'close'...
        );

% --- The following commands set up the common uicontrols

    top       = 0.15;       %Constants for placing objects
    leftSide  = 0.05;
    rightSide = 0.75;
    bottom    = 0.05;
    labelHt   = 0.05;
    spacing   = 0.005;
    frmBorder = 0.02;
    frmPos    = [0.05-0.02 0.05-0.02 ...
                .57+2*0.02 (0.15-0.05)+2*0.02];
    txtPos   =  [0.05-0.02 1-0.05 .57 0.05];
    
    textBox = ... % Status window
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'Max',             10,...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'Position',        [.03 .92 .57 .05]...
        );

    locationsText = ... % Selectable text for entering options menu
        uicontrol(mainWindow,...
            'Style',               'Text',...
            'String',              'Locations: ',...
            'HorizontalAlignment', 'Right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Units',               'normalized',...
            'Position',            [ .6 .94 .1 .03]...
    );

    optionsbutton = ... % button for entering options menu
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'String',          'Options',...
            'Units',           'normalized',...
            'Position',        [ .7 .94 .1 .03],...
            'visible', 'off',...
            'Callback', 'Asufit(''optionsScreen'');');

    plotbutton = ... % Selectable text for plotting data
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'String',          'Plot',...
            'Units',           'normalized',...
            'Position',        [ .8 .94 .1 .03],...
            'visible', 'off',...
            'Callback', 'Asufit(''plot'');');
         
    svdbutton = ... % Selectable text for plotting data
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'String',          'SVD',...
            'Units',           'normalized',...
            'Visible',         'off',...
            'Position',        [ .9 .94 .1 .03],...
    			'Callback', 'Asufit(''svdAnalysis'');');

    actionsText = ... % Selectable text for entering options menu
        uicontrol(mainWindow,...
            'Style',               'Text',...
            'String',              'Actions: ',...
            'HorizontalAlignment', 'Right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Units',               'normalized',...
            'Position',            [ .6 .91 .1 .03]...
    );

    action1button = ... % Selectable text for plotting data
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'Interruptible',   'On',...
            'Position',        [ .7 .91 .1 .03]);

    action2button = ... % Selectable text for plotting data
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'Interruptible',   'On',...
            'Position',        [ .8 .91 .1 .03]);

    action3button = ... % Selectable text for plotting data
        uicontrol(mainWindow,...
            'Style',           'pushbutton',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'Interruptible',   'On',...
            'Position',        [ .9 .91 .1 .03]);


    
% --- The following commands set up the uicontrols for SPC work
    spcSpectrumText = ... % Identifier for spectrum box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'Data: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .14 .1 .03]);
    

   spcSpectrumBox =...% Spectrum text box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'Units',               'normalized',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'position',            [.05 .11 .1 .03]...
        );

    spcPromptText = ... % Identifier for prompt box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'IRF: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.15 .14 .1 .03]);
        
    spcPromptBox = ... % Prompt text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.15 .11 .1 .03]);

    spcDateText = ... % Identifier for date text box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'String', 'Date: ',...
            'visible',             'off',...
            'Units',               'normalized',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.25 .14 .1 .03]);
        
    spcDateBox = ... % Date text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.25 .11 .1 .03]);

    spcExciteText = ... % Identifier for excite box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'String', 'Excitation: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.35 .14 .1 .03] );
         
         spcExciteBox = ... % Excitation text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'visible',         'off',...
            'Units',           'normalized',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.35 .11 .1 .03]);

    spcEmitText = ... % Identifier for emmission number box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'Detection: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.45 .14 .1 .03]);

     spcEmitBox = ... % Emmission text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.45 .11 .1 .03]...
        );

    spctpchText =  ... % Identifier for time/channel box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'time/channel: ', ...
            'visible',             'off',...
            'Units',               'normalized',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.55 .14 .1 .03]);
        
    spctpchBox = ... % tpch text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.55 .11 .1 .03]...
        );

    spcShiftNumText = ... % Identifier for shift box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'Shift: ', ...
            'Units',               'normalized',...
            'HorizontalAlignment', 'center',...
            'visible',             'off',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.65 .14 .1 .03]);

    spcShiftNumBox = ... % Shift text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.65 .11 .1 .03]);
    spcErrorText = ... % Idenfier for spc chi-squared box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'string', 'Chi^2: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.75 .14 .1 .03]);

    spcErrorBox = ... % SPC chi-squared text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.75 .11 .1 .03]);

    spcGlobalErrorText = ... % Identfier for spc global chi-squared box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'String', 'Global Chi^2: ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.85 .14 .1 .03]);
        
    spcGlobalErrorBox = ... % SPC global chi-squared box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'center',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.85 .11 .1 .03]);

    spcTaoAnsText = ... % Identifier for tao box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'String', 'Lifetimes (in nanoseconds): ', ...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .08 .25 .03]);
        
    spcTaoAnsBox = ... % Rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'Text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.3 .08 .85 .03]);

    spcAmpAnsText = ... % Identifier for amplitude box
        uicontrol(mainWindow,...
        'Style',               'text',...
        'String', 'Amplitudes (normalized to 100): ',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .05 .25 .03]);
        
    spcAmpAnsBox = ... % Amplitude display text box
        uicontrol(mainWindow,...
            'Style',           'Text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.3 .05 .85 .03]);

% ------------------------------------------------------------------------
% --- The following commands set up the uicontrols for ABSORBANCE work ---
% ------------------------------------------------------------------------        
    absCurrentWlText = ... % Identifier for current wavelength box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',             [.15 .87 .1 0.03]...
        );
        str='Wavelength: ';
        set(absCurrentWlText,'String',str);
    absCurrentWlBox = ... % Display/Input box for current wavelength
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'BackgroundColor', [.8 .8 .8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.25 .87 .1 0.03],...
            'CallBack',        'Asufit(''getWavelength'')'...
        );
    absCurrentTimeText = ... % Identifier for current time box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'BackgroundColor',     [.8 .8 .8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.65 .87  0.1 0.03]...
        );
        str='Time: ';
        set(absCurrentTimeText,'String',str);
    absCurrentTimeBox = ... % Display/input box for current time
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'BackgroundColor', [.8 .8 .8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.75 .87 .1 0.03],...
            'CallBack',        'Asufit(''getTime'')'...
        );
    absAbsorbText = ... % Identifier for absorbance box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'BackgroundColor',     [.8 .8 .8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.4 .87  0.1 0.03]...
        );
        str='Absorbance: ';
        set(absAbsorbText,'String',str);
    absAbsorbBox = ... % Absorbance display text box
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'BackgroundColor', [.8 .8 .8], ...
            'ForegroundColor', [0 0 0], ...
            'position',        [.5 .87 .1 0.03]....
        );
    absErrorText = ... % Identifier for absorbance chi-squared
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'BackgroundColor',     [.8 .8 .8], ...
            'ForegroundColor',     [0 0 0], ...
            'String', 'Chi-squared:',...
            'Position',            [.05 0.05 .1 0.03]...
        );
    absErrorBox = ... % Absorbance chi-squared display text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'BackgroundColor', [.8 .8 .8], ...
            'ForegroundColor', [0 0 0], ...
            'position',        [.15 .05 .1 0.03]...
    );
    absTaoAnsText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'String',				  {'taos: ';'delay zero: ';'FWHM of IRF: ';'Disp. param.: '},...
            'BackgroundColor',     [.8 .8 .8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.25 .0  0.2 0.08]....
        );
     absTaoAnsBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [.8 .8 .8], ...
            'ForegroundColor', [0 0 0], ...
            'position',        [.45 .0 .6 0.08]...
        );

% -----------------------------------------------------------------------
% --- The following commands set up the uicontrols for the SVD screen ---
% -----------------------------------------------------------------------
    svdTimeText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...      
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.05 .87  0.185 0.03]....
        );
        str='t: ';
        set(svdTimeText,'String',str);
    svdTimeBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'CallBack',        'Asufit(''svdGetTime'');',...      
            'BackgroundColor', [0.8 0.8 0.8], ...
            'ForegroundColor', [1 0 0], ...
            'position',        [.235 .87 .1 0.03]...
        );
    svdWavelengthText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.45 .87  0.185 0.03]...
        );
        str='wavelength: ';
        set(svdWavelengthText,'String',str);
    svdWavelengthBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'CallBack',        'Asufit(''svdGetWavelength'');',...      
            'BackgroundColor', [0.8 0.8 0.8], ...
            'ForegroundColor', [1 0 0], ...
            'position',        [.635 .87 .1 0.03]...
        );
    svdNumVecsText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...      
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.05 .39  0.1 0.03]....
        );
        str='Components: ';
        set(svdNumVecsText,'String',str);
    svdNumVecsBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Visible',         'off',...
            'Units',           'normalized',...
            'CallBack',        'Asufit(''svdGetNumVecs'');',...      
            'BackgroundColor', [0.8 0.8 0.8], ...
            'ForegroundColor', [0 0 0], ...
            'position',        [.15 .39 .1 0.03]...
        );
    svdErrorText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'right',...
            'Units',               'normalized',...      
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.05 .36  0.1 0.03]....
        );
        str='Error: ';
        set(svdErrorText,'String',str);
    svdErrorBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Visible',             'off',...
            'Units',               'normalized',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'position',            [.15 .36 .1 0.03]...
        );
    svdWeightText = ... % Identifier for absorbance rate constant box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'visible',             'off',...
            'HorizontalAlignment', 'center',...
            'Units',               'normalized',...
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'Position',            [.8 .84  0.2 0.03]....
        );
        str = 'Component weights';
        set(svdWeightText,'String',str);
    svdWeightBox = ... % Absorbance rate constant display text box
        uicontrol(mainWindow,...
            'Style',               'edit',...
            'Visible',             'off',...
            'Units',               'normalized',...
            'Max',                 10,...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8], ...
            'ForegroundColor',     [0 0 0], ...
            'position',            [.85 .05 .1 0.79]...
        );
    svdGlobalTogHypertext = ... 
        uicontrol(mainWindow,...
            'Style',               'Text',...
            'String',              'Global View',...
            'Visible',             'Off',...
            'HorizontalAlignment', 'Center',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Units',               'normalized',...
            'Position',            [ .05 .33 .2 .03]...
    );

% -------------------------------------------------------------------------------
% --- The following commands set up the uicontrols for the SPC options screen ---
% -------------------------------------------------------------------------------

    spcNumTaoText = ... % Identifier for number of rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Number of variable tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .35 .3 .03]...
        );
    spcNumTaoBox = ... % Input and display for number of rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .35 .6 .03],...
            'CallBack',        'Asufit(''getSpcOptions'');'...
        );
    spcTaoText = ... % Identifier for rate constants box
       uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Guesses for variable tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .32 .3 .03]...
        );
    spcTaoBox = ... % Input and display for rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .32 .6 .03],...
            'CallBack',        'Asufit(''getSpcOptions'')'...
        );
    spcNumTaoFixedText = ... % Identifier for number of fixed rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Number of fixed tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .29 .3 .03]...
        );
    spcNumTaoFixedBox = ... % Display for number of fixed rate constants
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.35 .29 .6 .03]...
        );
    spcTaoFixedText = ... % Identifier for fixed rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Values of fixed tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .26 .3 .03]...
        );
    spcTaoFixedBox = ... % Input and display for fixed rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .26 .6 .03],...
            'CallBack',        'Asufit(''getSpcOptions'')'...
        );
    spcNumAllTaoText = ... % Identifier for all rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Total number of tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .23 .3 .03]...
        );
    spcNumAllTaoBox = ... Display for all rate constants
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'position',            [.35 .23 .6 .03]...
        );
    spcAllTaoText = ... % Identifier for all rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Tao values are: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .2 .3 .03]...
        );
    spcAllTaoBox = ... Display for all rate constants
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'position',            [.35 .2 .6 .03]...
        );
    spcshifttext = ... % Identifier for x-tolerance box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Shift: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .17 .3 .03]...
        );
    spcshift = ... % Input and display for x-tolerance
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'Visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'Position',        [.35 .17 .6 .03],...
            'Callback',        'Asufit(''getSpcOptions'')');
     
     spcMaxItText = ... %Identifer for maximum number of iterations box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Maximum iterations: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .14 .3 .03]...
        );
    spcMaxItBox = ... % Input and display for maximum number of iterations
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .14 .6 .03],...
            'Callback',        'Asufit(''getSpcOptions'');'...
        );
   
% --------------------------------------------------------------------------------------
% --- The following commands set up the uicontrols for the ABSORBANCE options screen ---
% --------------------------------------------------------------------------------------

    absNumTaoText = ... % Identifier for number of rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Number of variable tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .38 .3 .03]...
        );
    absNumTaoBox = ... % Input and display for number of rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .38 .6 .03],...
            'CallBack',        'Asufit(''getAbsOptions'');'...
        );
    absTaoText = ... % Identifier for rate constants box
       uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Guesses for variable tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .35 .3 .03]...
        );
    absTaoBox = ... % Input and display for rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .35 .6 .03],...
            'CallBack',        'Asufit(''getAbsOptions'')'...
        );
    absNumTaoFixedText = ... % Identifier for number of fixed rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Number of fixed tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .32 .3 .03]...
        );
    absNumTaoFixedBox = ... % Display for number of fixed rate constants
        uicontrol(mainWindow,...
            'Style',           'text',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [0 0 0],...
            'position',        [.35 .32 .6 .03]...
        );
    absTaoFixedText = ... % Identifier for fixed rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Values of fixed tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .29 .3 .03]...
        );
    absTaoFixedBox = ... % Input and display for fixed rate constants
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .29 .6 .03],...
            'CallBack',        'Asufit(''getAbsOptions'')'...
        );
    absNumAllTaoText = ... % Identifier for all rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Total number of tao: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .26 .3 .03]...
        );
    absNumAllTaoBox = ... Display for all rate constants
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'position',            [.35 .26 .6 .03]...
        );
    absAllTaoText = ... % Identifier for all rate constants box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Tao values are: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .23 .3 .03]...
        );
    absAllTaoBox = ... Display for all rate constants
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'HorizontalAlignment', 'left',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'position',            [.35 .23 .6 .03]...
        );
     
     absDisperText = ...
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Dispersion Correction: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .17 .3 .03]...
        );
    absDisper1Hypertext = ... 
        uicontrol(mainWindow,...
            'Style',               'radiobutton',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'None',...
            'BackgroundColor',     [0.8 0.8 0.8],...            
            'Position',            [.35 .17 .1 .03],...
            'Max', 1,...
            'Min', 0,...
            'Callback',       'Asufit(''noDispersion'');'...
        );
    absDisper2Hypertext = ...
        uicontrol(mainWindow,...
            'Style',               'radiobutton',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'During fit',...
            'BackgroundColor',     [0.8 0.8 0.8],...            
            'Position',            [.45 .17 .1 .03],...
            'Max', 1, 'Min', 0,...
            'Callback',       'Asufit(''dispersionDuringFit'');'...
        );
      abscheckboxtext = ...
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Set constant:',...
            'BackgroundColor',     [0.8 0.8 0.8],...            
            'ForegroundColor',     [0 0 0],...
            'Position',            [.6 .17 .2 .03]);
      
      absDisperCoeffsText = ... % Identifier for
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              '[a,b,c] = ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .14 .3 .03]...
        );
    absDisperCoeffsBox = ... % Rate constant display text box
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .14 .3 .03],...
            'Callback',        'Asufit(''getAbsOptions'');'...
        );
     abscheckbox1 = ... 
        uicontrol(mainWindow,...
            'Style',           'checkbox',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'position',        [.7 .14 .03 .03],...
            'Callback',        'Asufit(''getAbsOptions'');'...
        );

        absPulseWidthText = ... % Identifier for pulsewidth box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'FWHM of IRF : ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .11 .3 .03]...
        );
    absPulseWidthBox = ... % Input and display for pulse width
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'Visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'Position',        [.35 .11 .3 .03],...
            'Callback',        'Asufit(''getAbsOptions'')'...
        );
    abscheckbox2 = ... 
        uicontrol(mainWindow,...
            'Style',           'checkbox',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'position',        [.7 .11 .03 .03],...
            'Callback',        'Asufit(''getAbsOptions'');'...
        );

        absShiftGuessText = ... % Identifier for delay zero
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Zero delay: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .08 .3 .03]...
        );
    absShiftGuessBox = ... % Input and display for delay zero
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .08 .3 .03],...
            'Callback',        'Asufit(''getAbsOptions'')'...
        );
   abscheckbox3 = ... 
        uicontrol(mainWindow,...
            'Style',           'checkbox',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'position',        [.7 .08 .03 .03],...
            'Callback',        'Asufit(''getAbsOptions'');'...
        );

        absMaxItText = ... %Identifer for maximum number of iterations box
        uicontrol(mainWindow,...
            'Style',               'text',...
            'Units',               'normalized',...
            'visible',             'off',...
            'String',              'Maximum iterations: ',...
            'HorizontalAlignment', 'right',...
            'BackgroundColor',     [0.8 0.8 0.8],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [.05 .05 .3 .03]...
        );
    absMaxItBox = ... % Input and display for maximum number of iterations
        uicontrol(mainWindow,...
            'Style',           'edit',...
            'Units',           'normalized',...
            'visible',         'off',...
            'HorizontalAlignment','left',...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'position',        [.35 .05 .3 .03],...
            'Callback',        'Asufit(''getAbsOptions'');'...
        );
    
    % --- The following commands display the configured Asufit figure window

    promptStr='Open file(s) to begin analysis.';
    set(textBox,'String',promptStr);
    set(mainWindow,'visible','on');

    % --- The followiwng commands check to see if data is present and acts
    % --- approptiately
    if (~isempty(analysisType))
        if (analysisType == 1)
            Asufit('spc')
        elseif (analysisType == 2)
            Asufit('abs')
        elseif (analysisType == 3)
           Asufit('svd')    
        end
    end

elseif strcmp(action,'spc')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets up the Asufit window for SPC data in the workspace
% for the SPC data present
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    % Enable SPC menu items
    
    set(expFile,'enable','off');
    set(kineticsFile,'enable','off');
    set(spectrumFile,'enable','off');
    set(plot3D,'enable','off');
    set(ampSpecPlot,'enable','off');
    set(ampSpecFile,'enable','off');    
    set(absDispersion,'enable','off');
    set(abstimeaxis,'enable','off');
    set(subtract,'enable','on');
    if (~isempty(fit))
        set(spcFitSave,'Enable','on');
    end
    if (isempty(spcLeftEnd))
        spcLeftEnd = 1;
    end
    if (isempty(spcRightEnd))
        spcRightEnd = length(t);
    end
    if (isempty(spcPlotMode))
        spcPlotMode = 1;
    end

    % Display text to command window

    promptStr='SPC/IRF data present in workspace.';
    set(textBox,'String',promptStr);
    set(optionsbutton,'visible', 'on');
    set(plotbutton,'Visible', 'on');
    set(svdbutton,'Visible', 'off');
    Asufit('setSpcOptions')

elseif strcmp(action,'abs')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument seta up the Asufit window for absorbance data in the
% workspace
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    % Enable absorbance menu items

    set(expFile,'enable','on');
    set(kineticsFile,'enable','on');
    set(spectrumFile,'enable','on');
    set(plot3D,'enable','on');    
    set(ampSpecFile,'enable','on');
    set(absDispersion,'enable','on');
    set(abstimeaxis,'enable','on');
    set(subtract,'Enable','On');
    set(restore,'Enable','On');

    % Activate absorbance controls
    
    if (isempty(spcLeftEnd))
        spcLeftEnd = 1;
    end
    if (isempty(spcRightEnd))
        spcRightEnd = length(t);
    end
    if (isempty(spcPlotMode))
        spcPlotMode = 1;
    end

    % Display text to command window

    set(optionsbutton,'Visible', 'on');
    set(plotbutton,'Visible', 'on');

    if (isempty(absLeftEnd1))
        absLeftEnd1 = 1;
    end
    if (isempty(absRightEnd1))
        absRightEnd1 = length(t);
    end
    if (isempty(absLeftEnd2))
        absLeftEnd2 = 1;
    end
    if (isempty(absRightEnd2))
        absRightEnd2 = length(wavelength);
    end

    % Display text to command window
    set(svdbutton,'Visible', 'on');
    set(action1button,'String','Fit');
    set(action1button,'Callback','Asufit(''fit'');');
    set(action1button,'Visible','on');
    set(action3button,'Visible','on');
    set(action3button,'String','Model');
    set(action3button,'Callback','Asufit(''selectfitmodel'');');
    set(action2button,'Visible','On');
    set(action2button,'String','DispCorr');
    set(action2button,'Callback','Asufit(''dispersionNow'');');
    Asufit('setAbsOptions')
    
elseif strcmp(action, 'allOff')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action turns off all figure items which are not common to absorbance and
% SPC anlysis sessions
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    Asufit('menuOff');
    Asufit('spcPlotOff');
    Asufit('spcOptionsOff');
    Asufit('absPlotOff');
    Asufit('absOptionsOff');
    Asufit('svdOff');
    
 elseif (strcmp(action, 'menuOff'))
    set(expFile,'enable','off')
    set(kineticsFile,'Enable','Off')
    set(spectrumFile,'Enable','Off')
    set(ampSpecFile,'Enable','Off')
    set(spcFitSave,'Enable','Off')
    set(plot3D,'Enable','Off')
    set(ampSpecPlot,'Enable','Off')
    set(subtract,'Enable','Off')
    set(spcGlobal,'Enable','Off')
    set(restore,'Enable','Off')
    set(abstimeaxis,'Enable', 'off')   

elseif (strcmp(action, 'spcPlotOff'))
    axes(spcDataGrid)
    cla
    set(spcDataGrid,'Visible','Off') 
    axes(spcResidGrid)
    cla
    set(spcResidGrid,'Visible','Off')
    set(spcSpectrumText,'Visible','Off')
    set(spcPromptText,'Visible','Off')
    set(spcDateText,'Visible','Off')
    set(spcSpectrumBox,'Visible','Off')
    set(spcPromptBox,'Visible','Off')
    set(spcDateBox,'Visible','Off')
    set(spcExciteText,'Visible','Off')
    set(spcEmitText,'Visible','Off')
    set(spctpchText,'Visible','Off')
    set(spcExciteBox,'Visible','Off')
    set(spcEmitBox,'Visible','Off')
    set(spctpchBox,'Visible','Off')
    set(spcShiftNumText,'Visible','Off')
    set(spcErrorText,'Visible','Off')
    set(spcGlobalErrorText,'Visible','Off')
    set(spcShiftNumBox,'Visible','Off')
    set(spcErrorBox,'Visible','Off')
    set(spcGlobalErrorBox,'Visible','Off')
    set(spcTaoAnsText,'Visible','Off')
    set(spcTaoAnsBox,'Visible','Off')
    set(spcAmpAnsText,'Visible','Off')
    set(spcAmpAnsBox,'Visible','Off')

elseif (strcmp(action, 'spcOptionsOff'))
    axes(spcIntSetAxes)
    cla
    set(spcIntSetAxes,'Visible','Off')
    set(spcNumTaoText,'Visible','Off')
    set(spcNumTaoBox,'Visible','Off')
    set(spcTaoText,'Visible','Off')
    set(spcTaoBox,'Visible','Off')
    set(spcNumTaoFixedText,'Visible','Off')
    set(spcNumTaoFixedBox,'Visible','Off')
    set(spcTaoFixedText,'Visible','Off')
    set(spcTaoFixedBox,'Visible','Off')
    set(spcNumAllTaoText,'Visible','Off')
    set(spcNumAllTaoBox,'Visible','Off')
    set(spcAllTaoText,'Visible','Off')
    set(spcAllTaoBox,'Visible','Off')    
    set(spcshifttext,'Visible','Off')
    set(spcshift,'Visible','Off')
    %set(spcFTolText,'Visible','Off')
   % set(spcFTolBox,'Visible','Off')
    set(spcMaxItText,'Visible','Off')
    set(spcMaxItBox,'Visible','Off')

   
elseif (strcmp(action, 'absPlotOff'))
    axes(absDataGrid1);
    cla;
    set(absDataGrid1,'Visible','Off')
    axes(absDataGrid2);
    cla;
    set(absDataGrid2,'Visible','Off')
    axes(absResidGrid1);
    cla;
    set(absResidGrid1,'Visible','Off')
    axes(absResidGrid2);
    cla;
    set(absResidGrid2,'Visible','Off')        
    set(absCurrentWlText,'Visible','Off')
    set(absCurrentWlBox,'Visible','Off')
    set(absCurrentTimeText,'Visible','Off')
    set(absCurrentTimeBox,'Visible','Off')
    set(absAbsorbText,'Visible','Off')
    set(absAbsorbBox,'Visible','Off')
    set(absTaoAnsText,'Visible','Off')
    set(absTaoAnsBox,'Visible','Off')
    set(absErrorText,'Visible','Off')
    set(absErrorBox,'Visible','Off')
    
elseif (strcmp(action, 'absOptionsOff'))    
    axes(absIntSetAxes1)
    cla
    set(absIntSetAxes1,'Visible','Off')
    axes(absIntSetAxes2)
    cla
    set(absIntSetAxes2,'Visible','Off')
    set(absNumTaoText,'Visible','Off')
    set(absNumTaoBox,'Visible','Off')
    set(absTaoText,'Visible','Off')
    set(absTaoBox,'Visible','Off')
    set(absNumTaoFixedText,'Visible','Off')
    set(absNumTaoFixedBox,'Visible','Off')
    set(absTaoFixedText,'Visible','Off')
    set(absTaoFixedBox,'Visible','Off')
    set(absNumAllTaoText,'Visible','Off')
    set(absNumAllTaoBox,'Visible','Off')
    set(absAllTaoText,'Visible','Off')
    set(absAllTaoBox,'Visible','Off')    
    set(absShiftGuessText,'Visible','Off')
    set(absShiftGuessBox,'Visible','Off')
%    set(absFTolText,'Visible','Off')
%   set(absFTolBox,'Visible','Off')
    set(absMaxItText,'Visible','Off')
    set(absMaxItBox,'Visible','Off')
    set(abscheckboxtext, 'Visible', 'Off')
    set(abscheckbox1, 'Visible', 'Off')
    set(abscheckbox2, 'Visible', 'Off')
    set(abscheckbox3, 'Visible', 'Off')
    set(absPulseWidthText,'Visible','Off')
    set(absPulseWidthBox,'Visible','Off')
%    set(absShiftGuessText,'Visible','Off')
%    set(absShiftGuessBox,'Visible','Off')
    set(absDisperCoeffsText,'Visible','Off')
    set(absDisperCoeffsBox,'Visible','Off')
    set(absDisperText,'Visible','Off')
    set(absDisper1Hypertext,'Visible','Off')
    set(absDisper2Hypertext,'Visible','Off')
 %   set(absDisper3Hypertext,'Visible','Off')
    
elseif (strcmp(action, 'svdOff'))
    axes(svdGrid1)
    cla
    set(svdGrid1,'Visible','Off')
    axes(svdGrid2)
    cla
    set(svdGrid2,'Visible','Off')
    axes(svdGrid3)
    cla
    set(svdGrid3,'Visible','Off')
    set(svdTimeText,'Visible','Off');
    set(svdTimeBox,'Visible','Off');
    set(svdWavelengthText,'Visible','Off');
    set(svdWavelengthBox,'Visible','Off');
    set(svdNumVecsText,'Visible','Off');
    set(svdNumVecsBox,'Visible','Off');
    set(svdErrorText,'Visible','Off');
    set(svdErrorBox,'Visible','Off');
    set(svdWeightText,'Visible','Off');
    set(svdWeightBox,'Visible','Off');
    set(svdGlobalTogHypertext,'Visible','Off');
    
elseif strcmp(action,'plot')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the current data and the fit if present
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    set(action1button,'Visible','off');
    set(svdbutton,'ForegroundColor',[1 0 0]); 
    set(optionsbutton,'ForegroundColor',[1 0 0]);
    set(plotbutton,'ForegroundColor',[0 1 0]);  
      if (analysisType == 1)
        set(spcSpectrumText,'Visible','on')
        set(spcSpectrumBox,'Visible','on')
        set(spcPromptText,'Visible','on')
        set(spcPromptBox,'Visible','on')
        set(spcDateText,'Visible','on')
        set(spcDateBox,'Visible','on')
        set(spcEmitText,'Visible','on')
        set(spcEmitBox,'Visible','on')
        set(spcExciteText,'Visible','on')
        set(spcExciteBox,'Visible','on')
        set(spctpchText,'Visible','on')
        set(spctpchBox,'Visible','on')
        set(spcDataGrid,'visible','on')

        if (isempty(spcInd))
            spcInd = 1;
        end

        Asufit('spcOptionsOff');
        set(spcShiftNumText,'visible','on');
        set(spcErrorText,'visible','on');
        set(spcGlobalErrorText,'visible','on');
        set(spcShiftNumBox,'visible','on');
        set(spcErrorBox,'visible','on');
        set(spcGlobalErrorBox,'visible','on');
        set(spcTaoAnsText,'visible','on');
        set(spcTaoAnsBox,'visible','on');
        set(spcAmpAnsText,'visible','on');
        set(spcAmpAnsBox,'visible','on');
        Asufit('spcPlot');
    elseif (analysisType == 2 | analysisType == 3)            
        Asufit('absOptionsOff');
        asufit('svdOff');
        set(absCurrentWlText,'Visible','On')
        set(absCurrentWlBox,'Visible','On')
        set(absCurrentTimeText,'Visible','On')
        set(absCurrentTimeBox,'Visible','On')
        set(absAbsorbText,'Visible','On')
        set(absAbsorbBox,'Visible','On')
        Asufit('absPlot')
    end
elseif (strcmp(action,'spcPlot'))
    set(action1button,'String','Previous');
    set(action1button,'Callback','Asufit(''spcPlotDown'');');
    set(action1button,'Visible','On');
    set(action2button,'String','Next');
    set(action2button,'Callback','Asufit(''spcPlotUp'');');
    set(action2button,'Visible','On');
    if (spcPlotMode == 1)
        str = 'Lin->Log';
    else
        str = 'Log->Lin';
    end
    set(action3button,'String',str);
    set(action3button,'Callback','Asufit(''spcChangePlotMode'');');
    set(action3button,'Visible','On');
    set(spcSpectrumBox,'String',spcFilenames(spcInd,:));
    set(spcPromptBox,'String',irfFilenames(spcInd,:));
    set(spcDateBox,'String',date(spcInd,:));
    set(spcEmitBox,'String',num2str(emit(spcInd,:)));
    set(spcExciteBox,'String',num2str(excite(spcInd,:)));
    set(spctpchBox,'String',num2str(tpch));
    if (spcPlotMode == 1)
       if (isempty(fit))
        set(spcErrorBox,'String','-');
        set(spcGlobalErrorBox,'String','-');
        set(spcShiftNumBox,'String','-');
        set(spcTaoAnsBox,'String','-');
        set(spcAmpAnsBox,'String','-');
        axes(spcDataGrid);
        spcPlot = plot(t(spcLeftEnd:spcRightEnd),data(spcLeftEnd:spcRightEnd,spcInd),'b','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) min(data(spcLeftEnd:spcRightEnd,spcInd)) max(data(spcLeftEnd:spcRightEnd,spcInd))]);
        axes(spcResidGrid);
        spcResidPlot = plot(t(spcLeftEnd:spcRightEnd),zeros(spcRightEnd - spcLeftEnd + 1,1),'k','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) -5 5]);
       else
        set(spcErrorBox,'String',num2str(chisq));
        set(spcGlobalErrorBox,'String',num2str(chisq));
        set(spcShiftNumBox,'String',num2str(shift(spcInd)));
        set(spcTaoAnsBox,'String',sprintf('%g  ',taoAns));
        set(spcAmpAnsBox,'String',sprintf('%g  ',amps(:,spcInd)));
        axes(spcDataGrid);
        spcPlot = plot(t(spcLeftEnd:spcRightEnd),data(spcLeftEnd:spcRightEnd,spcInd),'b',t(spcLeftEnd:spcRightEnd),fit(:,spcInd),'r','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) min(data(spcLeftEnd:spcRightEnd,spcInd)) max(data(spcLeftEnd:spcRightEnd,spcInd))]);
        axes(spcResidGrid);
        temp = (data(spcLeftEnd:spcRightEnd,spcInd)-fit(:,spcInd))./weight(spcLeftEnd:spcRightEnd,spcInd);
        spcResidPlot = plot(t(spcLeftEnd:spcRightEnd),zeros(spcRightEnd - spcLeftEnd + 1,1),'k',t(spcLeftEnd:spcRightEnd),temp,'r','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) min(temp) max(temp)]);
       end
    elseif (spcPlotMode == 2)
       if (isempty(fit))
        set(spcErrorBox,'String','-');
        set(spcGlobalErrorBox,'String','-');
        set(spcShiftNumBox,'String','-');
        set(spcTaoAnsBox,'String','-');
        set(spcAmpAnsBox,'String','-');
        axes(spcDataGrid);
        spcPlot = semilogy(t(spcLeftEnd:spcRightEnd),data(spcLeftEnd:spcRightEnd,spcInd),'b','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) 1 1e6]);
        axes(spcResidGrid);
        spcResidPlot = plot(t(spcLeftEnd:spcRightEnd),zeros(spcRightEnd - spcLeftEnd + 1,1),'k','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) -5 5]);
       else
        set(spcErrorBox,'String',num2str(chisq));
        set(spcGlobalErrorBox,'String',num2str(chisq));
        set(spcShiftNumBox,'String',num2str(shift(spcInd)));
        set(spcTaoAnsBox,'String',sprintf('%g  ',taoAns));
        set(spcAmpAnsBox,'String',sprintf('%g  ',amps(:,spcInd)));
        axes(spcDataGrid);
        spcPlot = semilogy(t(spcLeftEnd:spcRightEnd),data(spcLeftEnd:spcRightEnd,spcInd),'b',t(spcLeftEnd:spcRightEnd),fit(:,spcInd),'r','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) 1 1e6]);
        axes(spcResidGrid);
        temp = (data(spcLeftEnd:spcRightEnd,spcInd)-fit(:,spcInd))./weight(spcLeftEnd:spcRightEnd,spcInd);
        spcResidPlot = plot(t(spcLeftEnd:spcRightEnd),zeros(spcRightEnd - spcLeftEnd + 1,1),'k',t(spcLeftEnd:spcRightEnd),temp,'r','erasemode','xor');
        axis([t(spcLeftEnd) t(spcRightEnd) min(temp) max(temp)]);
       end 
    end

elseif (strcmp(action,'subtract'))
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument removes the current spectrum from all spectra
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if (analysisType == 2)
  temp = inputdlg('Enter the number of spectra for baseline correction:','Input for baseline correction',1,{'10'});
  temp= char(temp);
  temp=str2num(temp);
  databack = mean(data(:,1:temp),2)*ones(1,size(data,2));
  data=data-databack;
  asufit('setAbsOptions');
elseif (analysisType == 1)
  temp = inputdlg('Enter the number of channels for baseline correction:','Input for baseline correction',1,{'50'});
  temp= char(temp);
  temp=str2num(temp);
  databack = ones(size(data,1),1)*mean(data(1:temp,:),1);
  irfback = ones(size(irf,1),1)*mean(irf(1:temp,:),1);
  data=data-databack;
  irf=irf-irfback;
  asufit('setSpcOptions');
end

elseif strcmp(action,'print')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument prints the current data, and the fit if it exists
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (analysisType == 2 | analysisType == 3)
        Asufit('printAbs');
    elseif (analysisType == 1)
        Asufit('printSpc');
    end

elseif strcmp(action,'check')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument checks to see if all parameters necessary for fitting
% have been set
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (analysisType == 2)
        if ([tao taoFixed])
            str = 'Ready to begin analysis.';
        else 
            str = 'Set number of tao or input value for fixed tao';
        end
        set(textBox,'String',str)
    elseif ((analysisType == 2) | (analysisType == 3))
        if (~(isempty(numTao)) & ~(isempty(pulseWidth)))
                str=['Set number of t and pulse width.'];
        elseif (~(isempty(numTao)))
                str=['Set number of t.'];
        elseif (~(isempty(pulseWidth)))
                str=['Set pulse width.'];
        else 
                str=['Ready to begin analysis.'];
        end
        set(textBox,'String',str)
    end

elseif strcmp(action, 'stop')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument interrupts the fit in progress
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%    OPTIONS(14) = OPTIONS(10);

elseif strcmp(action, 'spcPlotUp')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the next SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    lengthEmit = length(emit);
    if (spcInd >= lengthEmit) % No next plot
        spcInd = lengthEmit;
    else
        spcInd = spcInd + 1;
     end
    str = ['Curve ' num2str(spcInd) ' of ' num2str(size(data,2)) '.'];
    set(textBox,'String',str);
    asufit('spcPlot');

elseif strcmp(action, 'spcPlotDown')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the previous SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (spcInd >= 2)
        spcInd = spcInd - 1;        
    else
        spcInd = 1;
     end
    str = ['Curve ' num2str(spcInd) ' of ' num2str(size(data,2)) '.'];
    set(textBox,'String',str);
    asufit('spcPlot');
    
elseif strcmp(action, 'getWavelength')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets the current wavelength to the wavelength closest
% to user input
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    newWl = str2num(get(absCurrentWlBox,'String'));
    tempWl = abs(wavelength(absLeftEnd2:absRightEnd2) - newWl);
    temp = find(tempWl == min(tempWl));
    if (temp)
        absInd1 = temp;
        Asufit('absPlot');
    end

elseif strcmp(action, 'getTime')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets the current time to the time closest to user input
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    newT = str2num(get(absCurrentTimeBox,'String'));
    tempT = abs(t(absLeftEnd1:absRightEnd1) - newT);
    temp = find(tempT == min(tempT));
    if (temp)
        absInd2 = temp;
        Asufit('absPlot');
    end

elseif strcmp(action, 'openSpc')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument attempts to open new SPC/IRF files
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
entryPath = pathname; % Remember current directory
spcFilenames = [];
irfFilenames = [];
excite = [];
emit = [];
date =[];
data =[];
irf = [];
set(mainWindow, 'Name', 'ASUFIT- data analysis software');
    str = 'Select files or press ''cancel'' to end.';
    set(textBox,'String',str);
    %This loop reads in pairs of SPC and IRF files
    [filename, pathname] = uigetfile([pathname '*.spc'],'Select SPC file:');
    if (filename ~= 0)
        while (filename ~= 0)
            name = [pathname filename];
            entryPath = pathname;            
            spcFilenames = strvcat(spcFilenames, filename);
            fid = fopen(name,'rt');
            fgetl(fid);
            tpch = fscanf(fid,'%g',1);
            fgetl(fid);
            excite = [excite; fscanf(fid,'%g',1)];
            emit = [emit; fscanf(fid,'%g',1)];
            fgetl(fid);
            fgetl(fid);
            fgetl(fid);
            fgetl(fid);
            date = strvcat(date,fscanf(fid,'%11c',1));
            numTimes = fscanf(fid,'%g',1);
            data = [data, fscanf(fid,'%g',numTimes)];
            fclose(fid);
            [filename,pathname] = uigetfile([pathname '*.irf'],['Select IRF file for ' filename]);
            name = [pathname filename];        
            irfFilenames = strvcat(irfFilenames, filename);
            fid = fopen(name,'rt');
            fgetl(fid);
            numTimes = fscanf(fid,'%g',1);
            irf = [irf, fscanf(fid,'%g',numTimes)];
            fclose(fid);
            [filename, pathname] = uigetfile([pathname '*.spc'],'Select next SPC file:');
        end
        t=0:tpch:tpch*(numTimes-1);    
        data = data(numTimes:-1:1,:);
        irf = irf(numTimes:-1:1,:);
        analysisType = 1;
        shift = zeros(1,size(data,2));
        maxIt = 100;
        spcLeftEnd = 1;
        spcRightEnd = numTimes;
        %calculating weight matrix:
        weight=data;
        weight=weight(:);
        k = find(weight == 0);
        weight(k) = 1;
        weight = reshape(weight,size(data,1),size(data,2));
        weight = sqrt(weight);
        numTao = 0;
        tao = [];
        fit = [];
        chisq = [];
        taoAns= [];
        amps= [];
        pathname = entryPath;
        Asufit('allOff');
        Asufit('spc');
    else
       pathname = entryPath;
    end
    
 elseif strcmp(action, 'openExpFile')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument attempts to open new EXP file
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
entryPath= pathname;
[filename,pathname] = uigetfile([pathname '*.exp'],'Select EXP file');
    if (filename~=0)
        name = [pathname filename];
        fid = fopen(name,'r');        
        fgetl(fid);
        fgetl(fid);
        fgetl(fid);
        dataAv = fread(fid,1,'short');
        numTimePoints1 = fread(fid,1,'short');
        numWl = fread(fid,1,'short');
        firstT = fread(fid,1,'double');
        timeStep = fread(fid,1,'double');
        firstWl = fread(fid,1,'double');
        wavelengthStep = fread(fid,1,'double');
        dataMax = fread(fid,1,'double');
        dataMin = fread(fid,1,'double');
        data = fread(fid,[numWl,numTimePoints1],'float');
        numTimePoints2 = fread(fid,1,'short');
        secondT = fread(fid,1,'double');
        timestep2 = fread(fid,1,'double');
        data2 = fread(fid,[numWl,numTimePoints2],'float');
        str= fgetl(fid);
        fclose(fid); 
        wavelength = [firstWl:wavelengthStep:(firstWl+wavelengthStep*(numWl-1))];
        t = [firstT:timeStep:(firstT+timeStep*(numTimePoints1-1))];
        abstimelin=1;
        if (numTimePoints2~=0)
           t=[t, (t(length(t))+timestep2):timestep2:(t(length(t))+timestep2*numTimePoints2)];
           data = [data, data2];
           set(abstimeaxis, 'Visible','On','Enable','On');
           abstimelin=1;
        end
        absFilename = filename;
        restoreName = name;
        set(mainWindow, 'Name', ['ASUFIT: ',name]);
        save datatemp wavelength t data dataMax dataMin;
        absYTicks(1)=dataMin;
        for i = 2:5
            absYTicks(i) = absYTicks(i-1) + (dataMax - dataMin)/4;
        end
        absYTicks = sort([absYTicks 0]);
        analysisType = 2;
        xTol = .01;
        fTol = .001;
        maxIt = 1000;
        dataBackup = data;
        fit=[];
        absInd1 = 1;
        absInd2 = 1;
        absLeftEnd1 = 1;
        absLeftEnd2 = 1;
        absRightEnd1 = length(t);
        absRightEnd2 = length(wavelength);
        dispersionStatus = 0;
        svdAnal=0;
        fitmodel=1;
        if(str~=-1)
           set(textBox,'String', str);
        else
           set(textBox,'String', 'Absorption data read successfully');
        end
        Asufit('allOff')
        Asufit('abs')
    else
        set(textBox,'String','Absorbance data import aborted.');
        pathname= entryPath;
     end
     
  elseif strcmp(action,'averageData')
     entryPath = pathname; % Remember current directory
     fcount=0;
     data=[];
     t=[];
     wavelength=[];
     str = 'Select files or press ''cancel'' to end.';
     set(textBox,'String',str);
    %This loop reads in EXP files for averaging
    [filename, pathname] = uigetfile([pathname '*.exp'],'Select EXP file to average or click cancel to stop:');
    if (filename ~= 0)
     while (filename ~= 0)
            name = [pathname filename];
            entryPath = pathname;            
            fid = fopen(name,'r');        
            fcount=fcount+1;
        fgetl(fid);
        fgetl(fid);
        fgetl(fid);
        dataAv(fcount) = fread(fid,1,'short');
        numTimePoints1(fcount) = fread(fid,1,'short');
        numWl(fcount) = fread(fid,1,'short');
        firstT(fcount) = fread(fid,1,'double');
        timeStep(fcount) = fread(fid,1,'double');
        firstWl(fcount) = fread(fid,1,'double');
        wavelengthStep(fcount) = fread(fid,1,'double');
        dataMax(fcount) = fread(fid,1,'double');
        dataMin(fcount) = fread(fid,1,'double');
        data(:,:,fcount) = fread(fid,[numWl(fcount),numTimePoints1(fcount)],'float');
        temp = fread(fid,1,'short');
        if (~isempty(temp))
           numTimePoints2(fcount) = temp;%fread(fid,1,'short');
           secondT(fcount) = fread(fid,1,'double');
           timestep2(fcount) = fread(fid,1,'double');
           data2(:,:,fcount) = fread(fid,[numWl(fcount),numTimePoints2(fcount)],'float');
        else
           numTimePoints2(fcount) = 0;
           secondT(fcount) = 0;
           timestep2(fcount) = 0;
           data2(:,:,fcount)=0;
        end
        fclose(fid);
        [filename, pathname] = uigetfile([pathname '*.exp'],'Select EXP file to average or click cancel to stop:');
     end
     if all(numTimePoints1==numTimePoints1(1)) & all(numWl==numWl(1)) & all(firstT==firstT(1))...
           & all (timeStep==timeStep(1)) & all(firstWl==firstWl(1)) & all(wavelengthStep==wavelengthStep(1)) ...
           & all(numTimePoints2==numTimePoints2(1)) & all(secondT==secondT(1)) & all(timestep2==timestep2(1))
        data=sum(data,3)/fcount;
        data2=sum(data2,3)/fcount;
        numTimePoints1=numTimePoints1(1);
        numTimePoints2=numTimePoints2(1);
        numWl=numWl(1);
        firstT=firstT(1);
        timeStep=timeStep(1);
        firstWl=firstWl(1);
        wavelengthStep=wavelengthStep(1);
        secondT=secondT(1);
        timestep2=timestep2(1);
        dataMax=dataMax(1);
        dataMin=dataMin(1);
        wavelength = [firstWl:wavelengthStep:(firstWl+wavelengthStep*(numWl-1))];
        t = [firstT:timeStep:(firstT+timeStep*(numTimePoints1-1))];
        abstimelin=1;
        if (numTimePoints2~=0)
           t=[t, (t(length(t))+timestep2):timestep2:(t(length(t))+timestep2*numTimePoints2)];
           data = [data, data2];
           set(abstimeaxis, 'Visible','On','Enable','On');
           abstimelin=1;
        end
        %absFilename = filename;
        %restoreName = name;
        set(mainWindow, 'Name', ['ASUFIT: average of ',num2str(fcount),' files']);
        save datatemp wavelength t data dataMax dataMin;
        absYTicks(1)=dataMin;
        for i = 2:5
            absYTicks(i) = absYTicks(i-1) + (dataMax - dataMin)/4;
        end
        absYTicks = sort([absYTicks 0]);
        analysisType = 2;
        xTol = .01;
        fTol = .001;
        maxIt = 1000;
        dataBackup = data;
        fit=[];
        absInd1 = 1;
        absInd2 = 1;
        absLeftEnd1 = 1;
        absLeftEnd2 = 1;
        absRightEnd1 = length(t);
        absRightEnd2 = length(wavelength);
        dispersionStatus = 0;
        str = 'Setting up absorbance related graphical interfaces...';
        set(textBox,'String',str);
        pathname=entryPath;
        Asufit('allOff')
        Asufit('abs')
     else
        pathname=entryPath;
        str = 'Averaging cannot be completed, files contain different delays or wavelengths!';
        set(textBox,'String',str);
     end
  else
     pathname=entryPath;
     str='Averaging aborted';
     set(textBox,'String',str);
  end
  
elseif strcmp(action,'setAbsOptions')
    Asufit('absPlotOff');
    Asufit('svdOff');
    set(absNumTaoText,'visible','on');
    set(absNumTaoBox,'visible','on');
    if (isempty(numTao))
       set(absNumTaoBox,'String','0');
    else
       set(absNumTaoBox,'String',num2str(numTao));
    end
    set(absTaoText,'visible','on');
    set(absTaoBox,'visible','on');
    if (~isempty(tao))
       set(absTaoBox,'String',num2str(tao));
    end   
    set(absNumTaoFixedText,'visible','on');
    set(absNumTaoFixedBox,'visible','on');
    if (isempty(numTaoFixed))
       set(absNumTaoFixedBox,'String','0');
    else
       set(absNumTaoFixedBox,'String',num2str(numTaoFixed));
    end
    set(absTaoFixedText,'visible','on');
    set(absTaoFixedBox,'visible','on');
    if (~isempty(taoFixed))
       set(absTaoFixedBox,'String',num2str(taoFixed));
    end
    set(absNumAllTaoText,'Visible','on')
    set(absNumAllTaoBox,'Visible','on')
    if (numTao + numTaoFixed)
        set(absNumAllTaoBox,'String',num2str(numTao + numTaoFixed));
    else
       set(absNumAllTaoBox,'String','0');
    end
    set(absAllTaoText,'visible','on');
    set(absAllTaoBox,'visible','on');
    if ([tao taoFixed])
        set(absAllTaoBox,'String',num2str(sort([tao taoFixed])));
    end
    set(absPulseWidthText,'Visible','on')
    set(absPulseWidthBox,'Visible','on')
    if (~isempty(pulseWidth))
       set(absPulseWidthBox,'String',num2str(pulseWidth))
    end
    set(absShiftGuessText,'Visible','On');
    set(absShiftGuessBox,'Visible','On');
    if (~isempty(tzero))
       set(absShiftGuessBox,'String',num2str(tzero));
    end
    set(abscheckboxtext, 'Visible', 'On')
    set(abscheckbox1, 'Visible', 'On')
    set(abscheckbox2, 'Visible', 'On')
    set(abscheckbox3, 'Visible', 'On')
    set(absMaxItText,'visible','on');
    set(absMaxItBox,'visible','on','String',num2str(maxIt));
    axes(absIntSetAxes1);
    absIntPlot1 = plot([t(absLeftEnd1) t(absLeftEnd1)],[min(min(data)) max(max(data))],'r',[t(absRightEnd1) t(absRightEnd1)],[min(min(data)) max(max(data))],'r',t,data,'b','EraseMode','xor','Clipping','on');
    set(absIntPlot1(1),'ButtonDownFcn','Asufit(''absLeftEndButton1'');');
    set(absIntPlot1(2),'ButtonDownFcn','Asufit(''absRightEndButton1'');');
    for i = 3:2+size(data,1)
        set(absIntPlot1(i),'ButtonDownFcn','Asufit(''absAxisButtonDown1'');');
    end
    set(absIntSetAxes1,'ButtonDownFcn','Asufit(''absAxisButtonDown1'');');
    absTog1 = 0;
    axes(absIntSetAxes2);
    absIntPlot2 = plot([wavelength(absLeftEnd2) wavelength(absLeftEnd2)],[min(min(data)) max(max(data))],'r',[wavelength(absRightEnd2) wavelength(absRightEnd2)],[min(min(data)) max(max(data))],'r',wavelength,data,'b','EraseMode','xor','Clipping','on');
    set(absIntPlot2(1),'ButtonDownFcn','Asufit(''absLeftEndButton2'');');
    set(absIntPlot2(2),'ButtonDownFcn','Asufit(''absRightEndButton2'');');
    for i = 3:2+size(data,2)
        set(absIntPlot2(i),'ButtonDownFcn','Asufit(''absAxisButtonDown2'');');
    end
    set(absIntSetAxes2,'ButtonDownFcn','Asufit(''absAxisButtonDown2'');');
    absTog2 = 0;
    set(mainWindow,'WindowButtonUpFcn',';');
    set(absDisperCoeffsText,'visible','on');
    set(absDisperCoeffsBox,'visible','on');
    if (~isempty(disperCoeffs))
       set(absDisperCoeffsBox,'String',num2str(disperCoeffs));
    end
    set(absDisperText,'Visible','On')
    set(absDisper1Hypertext,'Visible','On')
    set(absDisper2Hypertext,'Visible','On')
  % set(absDisper3Hypertext,'Visible','On')
    if (isempty(dispersionStatus))
        dispersionStatus = 0;
    end
    if (dispersionStatus == 0)
        set(absDisper1Hypertext,'Value',1);
        set(absDisper2Hypertext,'Value',0);
    elseif (dispersionStatus == 1)
        set(absDisper1Hypertext,'Value',0);
        set(absDisper2Hypertext,'Value',1);
    end
     
elseif strcmp(action,'getAbsOptions')
    numTao = str2num(get(absNumTaoBox,'String'));
    if (isempty(numTao))
       set(textBox,'String','set the number of tao');
    else
       set(absNumTaoBox, 'String',num2str(numTao));
    end
    tao= str2num(get(absTaoBox,'String'));
    if isempty(tao) & ~isempty(numTao)
        tao = logspace(0,numTao,numTao);
    end
          if length(tao) > numTao
             tao= tao(1:numTao);
          end
          if length(tao) < numTao
             tao = logspace(0,numTao,numTao);
          end
          tao = sort(tao);
          temp= sprintf('%g ',tao);
          set(absTaoBox,'String',temp);
    taoFixed = str2num(get(absTaoFixedBox,'String'));
    if (taoFixed)
        taoFixed = sort(taoFixed);
        numTaoFixed = length(taoFixed);
        set(absNumTaoFixedBox,'String',num2str(numTaoFixed));
        temp= sprintf('%g ',taoFixed);
        set(absTaoFixedBox,'String',temp);
    else
        numTaoFixed = 0;
        taoFixed = [];
        set(absNumTaoFixedBox,'String','0')
        %set(absTaoFixedBox,'String','-')
    end
    temp = [tao taoFixed];
    if (isempty(temp))
        set(absNumAllTaoBox,'String','0');
        %set(absAllTaoBox,'String','-');
    else
        temp = sort(temp);
        set(absNumAllTaoBox,'String',num2str(length(temp)));
        set(absAllTaoBox,'String',num2str(temp));
    end 
    dispersionStatus = get(absDisper2Hypertext,'Value'); 
    if (dispersionStatus == 0)
        set(absDisper1Hypertext,'Value',1);
        set(absDisper2Hypertext,'Value',0);
    else
        set(absDisper1Hypertext,'Value',0);
        set(absDisper2Hypertext,'Value',1);
    end
    disperCoeffs = str2num(get(absDisperCoeffsBox,'String'));       
    if (isempty(disperCoeffs))
       if wavelength >= 700
          disperCoeffs = [-5e-6 0.012 -6.5];
       else
          disperCoeffs = [-5e-5 0.06 -20];
       end
    elseif (length(disperCoeffs) < 3)
       disperCoeffs = [disperCoeffs zeros(1,3 - length(disperCoeffs))];
    elseif (length(disperCoeffs) > 3)
        disperCoeffs = disperCoeffs(1:3);
     end
    temp= sprintf('%g ',disperCoeffs);
    set(absDisperCoeffsBox,'String',temp);
	 pulseWidth = str2num(get(absPulseWidthBox,'String'));
    if (~isempty(pulseWidth))
       set(absPulseWidthBox,'String',num2str(pulseWidth));
    else
       pulseWidth= 0.3;
       set(absPulseWidthBox,'String',num2str(pulseWidth));
    end
    tzero = str2num(get(absShiftGuessBox,'String'));
    if (~isempty(tzero))
       set(absShiftGuessBox,'String',num2str(tzero));
    else
       tzero = 0;
       set(absShiftGuessBox,'String',num2str(tzero));
    end
    maxIt = str2num(get(absMaxItBox,'String'));
    if (isempty(maxIt))
        set(absMaxItBox,'String','-');
    else
        set(absMaxItBox,'String',num2str(maxIt));
    end
    fitconst= [0 0 0];    
       if (get(abscheckbox1, 'value')== 1)
          fitconst(1)= 1;
       end
       if (get(abscheckbox2, 'value') == 1)
          fitconst(2) = 1;
       end
       if (get(abscheckbox3, 'value') == 1)
          fitconst(3) = 1;
       end 
  elseif (strcmp(action, 'saveExpFile'))
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument attempts to save EXP file
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    [filename,pathname] = uiputfile([pathname '*.exp'],'Save EXP file:');
    if (filename~=0)
        name = [pathname filename];
        fid = fopen(name,'w');        
        fprintf(fid,'Femto File\n');
        fprintf(fid,'Experiment format\n');
        fprintf(fid,'\n');
        fwrite(fid,1,'short');
        if (isempty(numTimePoints2))|(absRightEnd1<=numTimePoints1)
           numTimePoints1 = length(t(absLeftEnd1:absRightEnd1));
           timeStep1 = t(absLeftEnd1 + 1) - t(absLeftEnd1);
           dataT1 = data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1);
 			  numTimePoints2= 0;
        elseif (absRightEnd1>numTimePoints1)&(absLeftEnd1<=numTimePoints1) 
           numTimePoints1 = length(t(absLeftEnd1:numTimePoints1));
           numTimePoints2 = length(t(numTimePoints1+1:absRightEnd1));
           timeStep1 = t(absLeftEnd1 + 1) - t(absLeftEnd1);
           timeStep2 = t(absRightEnd1) - t(absRightEnd1-1);
           secondT = t(numTimePoints1+1);
           dataT1 = data(absLeftEnd2:absRightEnd2,absLeftEnd1:numTimePoints1);
           dataT2 = data(absLeftEnd2:absRightEnd2,numTimePoints1+1:absRightEnd1);
        elseif (absRightEnd1>numTimePoints1)&(absLeftEnd1>numTimePoints1)
           numTimePoints1 = length(t(absLeftEnd1:absRightEnd1));
           timeStep1 = t(absLeftEnd1 + 1) - t(absLeftEnd1);
           dataT1 = data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1);
           numTimePoints2= 0;  
        end
        fwrite(fid,numTimePoints1,'short');
        numWl = length(wavelength(absLeftEnd2:absRightEnd2));
        fwrite(fid,numWl,'short');
        firstT = t(absLeftEnd1);
        fwrite(fid,firstT,'double');
        %timeStep1 = t(absLeftEnd1 + 1) - t(absLeftEnd1);
        fwrite(fid,timeStep1,'double');
        firstWl = wavelength(absLeftEnd2); 
        fwrite(fid,firstWl,'double');
        wavelengthStep = wavelength(absLeftEnd2 + 1) - wavelength(absLeftEnd2);
        fwrite(fid,wavelengthStep,'double');
        dataMax = max(max(data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1)));
        fwrite(fid,dataMax,'double');
        dataMin = min(min(data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1)));
        fwrite(fid,dataMin,'double');
        %dataT = data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1);
        count1 = fwrite(fid,dataT1,'float');
        count2= 0;
        if (~isempty(numTimePoints2) & (numTimePoints2 ~=0) )
           fwrite(fid,numTimePoints2,'short');
           fwrite(fid,secondT,'double');
           fwrite(fid,timeStep2,'double');
           count2=fwrite(fid,dataT2,'float');
        else
           fwrite(fid,0,'short');
           fwrite(fid,0,'double');
           fwrite(fid,0,'double');
        end
        temp = inputdlg('Would you like to add the comment to the file?','Please input the comment:',1);
        temp= char(temp);
         if (~isempty(temp))
           fprintf(fid,'%s',temp);
         end
        fclose(fid); 
        if count1+count2 == (numTimePoints1+numTimePoints2)*numWl
           str = ['Absorbance data saved to file', name];
           set(textBox,'String',str);
        end
    else 
        str = ['Absorbance data saving aborted'];
           set(textBox,'String',str);
    end
elseif (strcmp(action, 'saveAbsKinetic'))
    [filename,pathname]=uiputfile([pathname '*.kin'],['Save kinetic data and fit for wavelength ' num2str(wavelength(absInd1)) ' nm:']);
    if filename~=0
        name = [pathname filename];
        fid = fopen(name,'wt+');
        saveTemp(1,:) = t(absLeftEnd1:absRightEnd1);
        saveTemp(2,:) = data(absInd1,absLeftEnd1:absRightEnd1);
        if ~isempty(fit)
        	  saveTemp(3,:) = fit(absInd1 - absLeftEnd2 + 1,:);
        end   
          fprintf(fid,['Wavelength: ' num2str(wavelength(absInd1)) ' nm \n']);
        if isempty(fit)
           fprintf(fid,'Delay	   Data \n');
           fprintf(fid,'%6.3f    %6.5f\n',saveTemp);
        else
           fprintf(fid,'Delay	   Data        Fit \n');
           fprintf(fid,'%6.3f    %6.5f    %6.5f\n',saveTemp);
        end
        fclose(fid);
        str=['Data and fit at wavelength ' num2str(wavelength(absInd1)) ' nm saved to ' name];
        set(textBox,'String',str)
    end
    
 elseif (strcmp(action, 'saveAbsSpectrum'))    
    [filename,pathname]=uiputfile([pathname '*.wl'],['Save spectrum data and fit for delay ' num2str(t(absInd2)) ' ps:']);
    if filename~=0
        name = [pathname filename];
        fid = fopen(name,'wt+');
        saveTemp(1,:) = wavelength(absLeftEnd2:absRightEnd2);
        saveTemp(2,:) = data(absLeftEnd2:absRightEnd2,absInd2)';
        if ~isempty(fit)
           saveTemp(3,:) = fit(:,absInd2 - absLeftEnd1 + 1)';
        end   
        fprintf(fid,['Delay: ' num2str(t(absInd2)) ' ps \n']);
        if isempty(fit)
           fprintf(fid,'Wavelength Data \n');
           fprintf(fid,'%6.2f    %6.5f\n',saveTemp);
        else
           fprintf(fid,'Wavelength Data       Fit \n');
           fprintf(fid,'%6.2f    %6.5f    %6.5f\n',saveTemp);
        end
        fclose(fid);
        str=['Data and fit at delay ' num2str(t(absInd2)) ' ps saved to ' pathname];
        set(textBox,'String',str)
    end
    
elseif (strcmp(action, 'saveAbsAmpSpec'))
    [filename,pathname]=uiputfile([pathname '*.das'],['Save amplitudes as:']);
    if filename~=0
        name = [pathname filename];
        fid = fopen(name,'wt+');
        saveTemp= [0 taoAns'; wavelength(absLeftEnd2:absRightEnd2)' amps];
        for saveInd=0:length(taoAns)
           format(1+4*saveInd:4+4*saveInd)='%g\t';
        end
        format=[format '\n'];
           fprintf(fid,format,saveTemp');
        fclose(fid);
        str = 'Amplitude data saved.';
        set(textBox,'String',str);
    end
    
elseif strcmp(action,'setSpcOptions')
    axes(spcDataGrid)
    cla
    set(spcDataGrid,'Visible','Off')
    axes(spcResidGrid)
    cla
    set(spcResidGrid,'Visible','Off')
    set(spcShiftNumText,'visible','off');
    set(spcShiftNumBox,'visible','off');
    set(spcErrorText,'visible','off');
    set(spcErrorBox,'visible','off');
    set(spcGlobalErrorText,'visible','off');
    set(spcGlobalErrorBox,'visible','off');
    set(spcShiftNumText,'visible','off');
    set(spcShiftNumBox,'visible','off');
    set(spcTaoAnsText,'visible','off');
    set(spcTaoAnsBox,'visible','off');
    set(spcAmpAnsText,'visible','off');
    set(spcAmpAnsBox,'visible','off');

    set(spcSpectrumText,'Visible','off')
    set(spcSpectrumBox,'Visible','off')
    set(spcPromptText,'Visible','off')
    set(spcPromptBox,'Visible','off')
    set(spcDateText,'Visible','off')
    set(spcDateBox,'Visible','off')
    set(spcEmitText,'Visible','off')
    set(spcEmitBox,'Visible','off')
    set(spcExciteText,'Visible','off')
    set(spcExciteBox,'Visible','off')
    set(spctpchText,'Visible','off')
    set(spctpchBox,'Visible','off')
   
    set(spcNumTaoText,'visible','on');
    set(spcNumTaoBox,'visible','on');
    if (numTao)
        str = num2str(numTao);
    else
        str = '0';
    end
    set(spcNumTaoBox,'String',str);
    set(spcTaoText,'visible','on');
    set(spcTaoBox,'visible','on');
    if (tao)
        str = sprintf('%g ',tao);
    else
        str = '-';
    end
    set(spcTaoBox,'String',str);
    set(spcNumTaoFixedText,'visible','on');
    set(spcNumTaoFixedBox,'visible','on');
    if (numTaoFixed)
        str = num2str(numTaoFixed);
    else
        str = '0'; 
    end
    set(spcNumTaoFixedBox,'String',str);
    set(spcTaoFixedText,'visible','on');
    set(spcTaoFixedBox,'visible','on');
    if (taoFixed)
        str = sprintf('%g ',taoFixed);
    else
        str = '-';
    end
    set(spcTaoFixedBox,'String',str);
    set(spcNumAllTaoText,'Visible','on')
    set(spcNumAllTaoBox,'Visible','on')
    if (numTao + numTaoFixed)
        str = num2str(numTao + numTaoFixed);
    else
        str = '0';
    end
    set(spcNumAllTaoBox,'String',str);
    set(spcAllTaoText,'visible','on');
    set(spcAllTaoBox,'visible','on');
    if ([tao taoFixed])
        str = sprintf('%g ',sort([tao taoFixed]));
    else
        str = '-';
    end
    set(spcAllTaoBox,'String',str);            
    set(spcshifttext,'visible','on');
    if (isempty(shift))
       shift = zeros(1,size(data,2));
       set(spcshift,'visible','on','String',num2str(shift));
    else
       set(spcshift,'visible','on','String',num2str(shift));
    end
    set(spcMaxItText,'visible','on');
    set(spcMaxItBox,'visible','on','String',num2str(maxIt));
    axes(spcIntSetAxes);
    spcIntPlot = semilogy([t(spcLeftEnd) t(spcLeftEnd)],[1 max(max(data))],'r-',[t(spcRightEnd) t(spcRightEnd)],[1 max(max(data))],'r-',t,irf,'k',t,data,'EraseMode','xor','Clipping','on');
    axis([t(1) t(length(t))  1 1e6]);
    set(spcIntPlot(1),'ButtonDownFcn','Asufit(''spcLeftEndButton'');');
    set(spcIntPlot(2),'ButtonDownFcn','Asufit(''spcRightEndButton'');');
    for i = 3:2+size(data,2)
        set(spcIntPlot(i),'ButtonDownFcn','Asufit(''spcAxisButtonDown'');');
    end
    set(spcIntSetAxes,'ButtonDownFcn','Asufit(''spcAxisButtonDown'');');
    set(mainWindow,'WindowButtonUpFcn','Asufit(''spcButtonUp'');');
    spcTog = 0;
    
elseif strcmp(action,'getSpcOptions')
    temp = str2num(get(spcNumTaoBox,'String'));
    if (numTao == temp)
        temp = str2num(['[' get(spcTaoBox,'String') ']']);
        temp = sort(temp);
        taoChanged = 0;
        if (isempty(tao) & isempty(temp))
            %Nothing, this is here to appease MATLAB
        elseif (length(tao) == length(temp))
            if (~all(tao == temp))
                taoChanged = 1;
            end
        else
            taoChanged = 1;
        end
        if (taoChanged)
            tao = temp;
            temp = sprintf('%.4g ',tao);
            set(spcTaoBox,'String',temp);
            numTao = length(tao);
            set(spcNumTaoBox, 'String',num2str(numTao));
        end
    else        
        numTao = temp;
        tao = zeros(1,numTao);
        if (~isempty(tao))
            tao = logspace(-2,1,numTao);
            temp = sprintf('%g ',tao);
            set(spcTaoBox,'String',temp);
        else
            set(spcTaoBox,'String','-');
        end
    end
    taoFixed = str2num(['[' get(spcTaoFixedBox,'String') ']']);
    if (taoFixed)
        taoFixed = sort(taoFixed);
        numTaoFixed = length(taoFixed);
        set(spcNumTaoFixedBox,'String',num2str(numTaoFixed));
        temp = sprintf('%g ',taoFixed);
        set(spcTaoFixedBox,'String',temp);
    else
        numTaoFixed = 0;
        taoFixed = [];
        set(spcNumTaoFixedBox,'String','0')
        set(spcTaoFixedBox,'String','-')
    end
    temp = [tao taoFixed];
    if (isempty(temp))
        set(spcNumAllTaoBox,'String','0');
        set(spcAllTaoBox,'String','-');
    else
        temp = sort(temp);
        set(spcNumAllTaoBox,'String',num2str(length(temp)));
        temp = sprintf('%f ',temp);
        set(spcAllTaoBox,'String',temp);
    end        
    shift = str2num(get(spcshift,'String'));
    if (isempty(shift))
        set(spcshift,'String','-');
     else
        temp= sprintf('%g ',shift);
        set(spcshift,'String',temp);
    end
    maxIt = str2num(get(spcMaxItBox,'String'));
    if (isempty(maxIt))
        set(spcMaxItBox,'String','-');
    else
        set(spcMaxItBox,'String',num2str(maxIt));
    end    

elseif strcmp(action, 'optionsScreen')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument changes the ASUFIT window into the options screen
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    set(optionsbutton,'ForegroundColor',[0 1 0]);
    set(plotbutton,'ForegroundColor',[1 0 0]);
    set(svdbutton,'ForegroundColor',[1 0 0]);
    set(action1button,'String','Fit');
    set(action1button,'Callback','Asufit(''fit'');');
    set(action1button,'Visible','on', 'ForegroundColor', [1 0 0]);
    set(action3button,'Visible','Off');
    if (analysisType == 1)
       set(action2button,'Visible','Off');
        Asufit('setSpcOptions');
     else
        set(action2button,'Visible','On');
        set(action2button,'String','DispCorr');
        set(action2button,'Callback','Asufit(''dispersionNow'');');
        set(action3button,'Visible','on');
        set(action3button,'String','Model');
        set(action3button,'Callback','Asufit(''selectfitmodel'');');
        Asufit('setAbsOptions');
    end

elseif strcmp(action, 'fit')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument does the fit
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (analysisType == 1)
        if (isempty(tao) & (spcRightEnd - spcLeftEnd + 1) >= length(t))
            str = 'Set the number of tao and the fit interval.';
            set(textBox,'String',str);
        elseif((spcRightEnd - spcLeftEnd + 1) > length(t))
            str = 'Set the fit interval.';
            set(textBox,'String',str);
        elseif(isempty(tao))
            str = 'Set the number of tao.';
            set(textBox,'String',str);
        else
            str = 'Initiating the fit.';
            set(textBox,'String',str);
            set(action1button,'String','Stop','Callback','Asufit(''StopFit'');'); 
            drawnow;
            Asufit('spcFit');
        end
    else        
        if (isempty([tao taoFixed]))
            str = 'Set the number of tao or input value for fixed tao.';
            set(textBox,'String',str);
        elseif (abstimelin == 0)
            msgbox('Set the time scale to linear!');
        else
            str = 'Initiating the fit.';
            set(textBox,'String',str);
            set(action1button,'String','Stop','Callback','Asufit(''StopFit'');');
            drawnow;
            Asufit('absFit');
        end
    end
    
elseif strcmp(action, 'spcIntMove')
    if (spcTog == 1)
        pt = get(spcIntSetAxes,'CurrentPoint');
        pt = find(t <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            if (pt < spcRightEnd)
                spcLeftEnd = pt;
                set(spcIntPlot(1),'XData',[t(spcLeftEnd) t(spcLeftEnd)]);
            end
        end
    elseif (spcTog == 2)
        pt = get(spcIntSetAxes,'currentpoint');
        pt = find(t >= pt(1,1));
        if (pt)
            pt = pt(1);
            if (pt > spcLeftEnd)
                spcRightEnd = pt;
                set(spcIntPlot(2),'XData',[t(spcRightEnd) t(spcRightEnd)]);
            end
        end
    end
elseif strcmp(action, 'spcLeftEndButton')
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');
        set(spcGlobal,'Enable','Off');
    end
    spcTog = 1;
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''spcIntMove'');');
    
 elseif strcmp(action, 'spcRightEndButton')
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');
        set(spcGlobal,'Enable','Off');
    end
    spcTog = 2;
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''spcIntMove'');');
    
 elseif strcmp(action, 'spcAxisButtonDown')
    pt = get(spcIntSetAxes,'currentpoint');
    pt = find(t <= pt(1,1));
    if (pt)
        pt = pt(length(pt));
        if (abs(spcRightEnd - pt) < abs(pt - spcLeftEnd))
            spcRightEnd = pt;
            set(spcIntPlot(2),'XData',[t(spcRightEnd) t(spcRightEnd)]);
            spcTog = 2;        
        else
            spcLeftEnd = pt;
            set(spcIntPlot(1),'XData',[t(spcLeftEnd) t(spcLeftEnd)]);
            spcTog = 1;
        end
        set(mainWindow,'WindowButtonMotionFcn','Asufit(''spcIntMove'');');
        if (~isempty(fit))
            fit = [];
            set(textBox,'String','Interval changed, fit cleared.');
            set(spcGlobal,'Enable','Off');
        end
    end

elseif strcmp(action, 'spcButtonUp')
    spcTog = 0;
    set(mainWindow,'WindowButtonMotionFcn',';');

elseif strcmp(action,'spcFit')
      %---Set options matrix---
%    OPTIONS = optimset('lsqnonlin');
    OPTIONS = optimset('Display', 'off', 'MaxIter', maxIt);
    %---Fit the data---
    set(spcIntSetAxes,'ButtonDownFcn',';');
    for i = 3:2+size(data,2)
        set(spcIntPlot(i),'ButtonDownFcn',';');
    end
    set(action1button,'String','Stop','Callback','Asufit(''StopFit'');');
    taoAns = zeros(1,numTao + numTaoFixed);
    params = [tao shift];
    params = lsqnonlin('spcfun',params,zeros(size(tao)),[],OPTIONS);
    set(action1button,'String','Fit','Callback','Asufit(''fit'');');
    taoAns(1:numTaoFixed) = taoFixed;
    taoAns(numTaoFixed+1:numTao+numTaoFixed) = params(1:numTao);
    tao = params(1:numTao);
    shift = params(1+numTao:numTao+size(data,2));
    amps=amps/max(sum(amps))*100;
    str = 'Fitting Complete.';
    set(textBox,'String',str);
    set(spcGlobal,'Enable','On');
    set(spcFitSave, 'Enable', 'On');
    asufit('plot');
    
elseif strcmp(action, 'spcStopFit')
    set(action1button,'ForegroundColor',[0 1 0]);
    OPT_STOP = 1;

elseif (strcmp(action, 'spcChangePlotMode'))
    if (spcPlotMode == 1)
        spcPlotMode = 2;
    else
        spcPlotMode = 1;
    end
    Asufit('spcPlot');
elseif (strcmp(action, 'spcFitSave'))
    thisSpectrum = spcFilenames(spcInd,:);
    entryPath = pathname;
    [filename,pathname]=uiputfile([pathname '*.fit'],['Save fit for ' thisSpectrum ':']);
    if (filename ~= 0)
        filename =[pathname filename];
        outFile = fopen(filename,'wt+');
        temp = (data(spcLeftEnd:spcRightEnd,spcInd)-fit(:,spcInd))./weight(spcLeftEnd:spcRightEnd,spcInd);
        fprintf(outFile,['/*Emission wavelength: ' num2str(emit(spcInd,:)) ' */ \n']); 
        fprintf(outFile,['/* Lifetimes (in nanoseconds): ' num2str(taoAns) ' */ \n']);
        fprintf(outFile,['/* Amplitudes (normalized to 100): ' num2str(amps(:,spcInd)') ' */ \n']);
        fprintf(outFile,'/* Time         Data          Fit           Residuals*/\n');
        fprintf(outFile,'%g        %g         %g         %g\n',...
           [t(spcLeftEnd:spcRightEnd); data(spcLeftEnd:spcRightEnd,spcInd)'; fit(:,spcInd)'; temp']);
        fclose(outFile);
        str=['Data and fit saved to ' filename];
        set(textBox,'String',str);
     else
        set(textBox,'String','save aborted');
        pathname=entryPath;
    end
    
elseif (strcmp(action, 'spcGlobalResultWindow'))
    if (~isempty(fit))
            %New temorary figure
            figure( ...
            'Name','Global fitting results', ...
            'Units','normalized',...
            'Position',[0.1 0.1 0.8 0.8],...
            'NumberTitle','off', ...
            'Visible','on');
         plot(emit,amps');
            end

elseif (strcmp(action, 'printSpc'))    
   print
   
elseif(strcmp(action, 'noDispersion'))
   dispersionStatus = 0; 
   set(absDisper1Hypertext,'Value',1);
   set(absDisper2Hypertext,'Value',0);
   Asufit('getAbsOptions');    
   
elseif(strcmp(action, 'dispersionDuringFit'))
   dispersionStatus = 1;
   set(absDisper1Hypertext,'Value',0);
   set(absDisper1Hypertext,'Value',1);
   Asufit('getAbsOptions');
         
elseif(strcmp(action, 'dispersionNow'))
    if (any(disperCoeffs))
       button = questdlg('WARNING!  Completing this command will change the workspace data.  Are you sure you wish to continue?',...
     'Continue Operation','Yes','No','Yes');
     if strcmp(button,'Yes')
        fit=[];
        Asufit('dispersionCorrection');
     elseif strcmp(button,'No')
        msgbox('Dispersion correction canceled.');
        Asufit('getAbsOptions');
     end
    else
       set(textBox,'String','Please input dispersion parameters');                 
    end   
elseif(strcmp(action, 'absLeftEndButton1'))
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');        
    end
    absTog1 = 1;
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove1'');');

elseif(strcmp(action, 'absRightEndButton1'))
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');
    end
    absTog1 = 2;
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove1'');');

elseif(strcmp(action, 'absAxisButtonDown1'))
    pt = get(absIntSetAxes1,'currentpoint');
    pt = find(t <= pt(1,1));
    if (pt)
        pt = pt(length(pt));
        if (abs(absRightEnd1 - pt) < abs(pt - absLeftEnd1))
            absRightEnd1 = pt;
            set(absIntPlot1(2),'XData',[t(absRightEnd1) t(absRightEnd1)]);
            absTog1 = 2;        
        else
            absLeftEnd1 = pt;
            set(absIntPlot1(1),'XData',[t(absLeftEnd1) t(absLeftEnd1)]);
            absTog1 = 1;
        end
        set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove1'');');
        set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
        if (~isempty(fit))
            fit = [];
            set(textBox,'String','Interval changed, fit cleared.');
        end
    end

elseif(strcmp(action, 'absLeftEndButton2'))
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');        
    end
    absTog2 = 1;
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove2'');');

elseif(strcmp(action, 'absRightEndButton2'))
    if (~isempty(fit))
        fit = [];
        set(textBox,'String','Interval changed, fit cleared.');
    end
    absTog2 = 2;
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove2'');');

elseif(strcmp(action, 'absAxisButtonDown2'))
    pt = get(absIntSetAxes2,'currentpoint');
    pt = find(wavelength <= pt(1,1));
    if (pt)
        pt = pt(length(pt));
        if (abs(absRightEnd2 - pt) <= abs(pt - absLeftEnd2))
            absRightEnd2 = pt;
            set(absIntPlot2(2),'XData',[wavelength(absRightEnd2) wavelength(absRightEnd2)]);
            absTog2 = 2;        
        else
            absLeftEnd2 = pt;
            set(absIntPlot2(1),'XData',[wavelength(absLeftEnd2) wavelength(absLeftEnd2)]);
            absTog2 = 1;
        end
        set(mainWindow,'WindowButtonMotionFcn','Asufit(''absIntMove2'');');
        set(mainWindow,'WindowButtonUpFcn','Asufit(''absButtonUp'');');
        if (~isempty(fit))
            fit = [];
            set(textBox,'String','Interval changed, fit cleared.');
        end
    end

elseif (strcmp(action, 'absIntMove1'))
    if (absTog1 == 1)
        pt = get(absIntSetAxes1,'currentpoint');
        pt = find(t <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            if (pt < absRightEnd1)
                absLeftEnd1 = pt;
                set(absIntPlot1(1),'XData',[t(absLeftEnd1) t(absLeftEnd1)]);
            end
        end
    elseif (absTog1 == 2)
        pt = get(absIntSetAxes1,'currentpoint');
        pt = find(t >= pt(1,1));
        if (pt)
            pt = pt(1);
            if (pt > absLeftEnd1)
                absRightEnd1 = pt;
                set(absIntPlot1(2),'XData',[t(absRightEnd1) t(absRightEnd1)]);
            end
        end
    end
elseif (strcmp(action, 'absIntMove2'))
    if (absTog2 == 1)
        pt = get(absIntSetAxes2,'currentpoint');
        pt = find(wavelength <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            if (pt < absRightEnd2)
                absLeftEnd2 = pt;
                set(absIntPlot2(1),'XData',[wavelength(absLeftEnd2) wavelength(absLeftEnd2)]);
            end
        end
    elseif (absTog2 == 2)
        pt = get(absIntSetAxes2,'currentpoint');
        pt = find(wavelength >= pt(1,1));
        if (pt)
            pt = pt(1);
            if (pt > absLeftEnd2)
                absRightEnd2 = pt;
                set(absIntPlot2(2),'XData',[wavelength(absRightEnd2) wavelength(absRightEnd2)]);
            end
        end
    end
elseif (strcmp(action, 'absButtonUp'))
    absTog1 = 0;
    absTog2 = 0;
    set(mainWindow,'WindowButtonMotionFcn',';');
    set(mainWindow,'WindowButtonUpFcn',';');
    Asufit('getAbsOptions');
    
elseif (strcmp(action, 'absFit'))
    %---Set options matrix---
%    OPTIONS = optimset('lsqnonlin');
    OPTIONS = optimset('Display', 'off', 'MaxIter', maxIt);
    %---Fit the data---
    set(absIntSetAxes1,'ButtonDownFcn',';');
    for i = 3:2+size(data,1)
        set(absIntPlot1(i),'ButtonDownFcn',';');
    end
    set(absIntSetAxes2,'ButtonDownFcn',';');
    for i = 3:2+size(data,2)
        set(absIntPlot2(i),'ButtonDownFcn',';');
    end
    taoAns = zeros(numTao + numTaoFixed,1);
    params=[];
    if fitmodel ==1 
      if (dispersionStatus ~= 1)
       if (fitconst(2) == 0 & fitconst(3) == 0)
          params = [tao tzero pulseWidth];
          params= lsqnonlin('absfunfree',params,zeros(size(tao)),[],OPTIONS);
          %[params,resnorm,residual,exitflag,output,lambda,jacobian] = lsqnonlin('absfunfree',params,zeros(size(tao)),[],OPTIONS);
          %sigma = sqrt(resnorm/(numPoints-numparams)/diag(jacobian'*jacobian));
          tao = params(1:numTao);
          tzero= params(numTao+1);
          pulseWidth= params(numTao+2);
       elseif (fitconst(2) == 1 & fitconst(3) == 0)
          params = [tao tzero];
          params = lsqnonlin('absfunshift',params,zeros(size(tao)),[],OPTIONS);
          tao = params(1:numTao);
          tzero= params(numTao+1);
       elseif (fitconst(2) == 0 & fitconst(3) == 1)
          params = [tao pulseWidth];
          params = lsqnonlin('absfunwidth',params,zeros(size(tao)),[],OPTIONS);
          tao = params(1:numTao);
          pulseWidth= params(numTao+1);
       else   
          params = [tao];
          params = lsqnonlin('absfun',params,zeros(size(tao)),[],OPTIONS);
          tao = params(1:numTao);
       end   
      else
        if (fitconst(1) == 0 & fitconst(3) == 0)
           params = [tao disperCoeffs pulseWidth];
           params = lsqnonlin('dispfitfree',params,zeros(size(tao)),[],OPTIONS);
           tao = params(1:numTao);
           disperCoeffs= params(numTao+1:numTao+3);
           pulseWidth= params(numTao+4);
        elseif (fitconst(1) == 1 & fitconst(2) == 0)
           params = [tao tzero pulseWidth];
           params = lsqnonlin('dispfitwidth',params,zeros(size(tao)),[],OPTIONS);
           tao = params(1:numTao);
           tzero= params(numTao+1);
           pulseWidth= params(numTao+2);
        elseif (fitconst(1) == 0 & fitconst(2) == 1) 
           params = [tao disperCoeffs];
           params = lsqnonlin('dispfitdisp',params,zeros(size(tao)),[],OPTIONS);
           tao = params(1:numTao);
           disperCoeffs= params(numTao+1:numTao+3);
        else
           params = [tao tzero];
           params = lsqnonlin('dispfit',params,zeros(size(tao)),[],OPTIONS);
           tao = params(1:numTao);
           tzero= params(numTao+1);
        end
      end
  elseif fitmodel == 2
      params = [tao tzero pulseWidth];
      params= lsqnonlin('seqmodfree',params,zeros(size(tao)),[],OPTIONS);
          tao = params(1:numTao);
          tzero= params(numTao+1);
          pulseWidth= params(numTao+2);
      end
    fitError = fit - data(absLeftEnd2:absRightEnd2,absLeftEnd1:absRightEnd1);
    chisq = trace(fitError'*fitError);    
    set(action1button,'String','Fit','Callback','Asufit(''fit'');');
    paramsAns = params;
    taoAns(1:numTaoFixed) = taoFixed;
    taoAns(numTaoFixed+1:numTao+numTaoFixed) = paramsAns(1:numTao);
    tao=paramsAns(1:numTao);
    set(ampSpecPlot,'enable','on');	
    %---Plot the data (y) and the fit (z)---
    asufit('plot');
elseif (strcmp(action, 'StopFit'))
   OPT_STOP = 1;
   
elseif (strcmp(action,'absPlot'))
    if (isempty(absPlotKinSpec))
        absPlotKinSpec = 1;
    end    
    if (isempty(absInd1))    
        absInd1 = absLeftEnd2;
    elseif  (absInd1 < absLeftEnd2)
      absInd1 = absLeftEnd2;
    elseif (absInd1 > absRightEnd2)
       absInd1 = absRightEnd2;
    end
    if (isempty(absInd2))
        absInd2 = absLeftEnd1;
    elseif (absInd2 < absLeftEnd1)
       absInd2 = absLeftEnd1;
    elseif (absInd2 > absRightEnd1)
      absInd2 = absRightEnd1;
    end    
    set(action1button,'String','Previous');
    set(action1button,'Callback','Asufit(''absPlotDown'');');
    set(action1button,'Visible','On');
    set(action1button,'Foregroundcolor',[1 0 0]);
    set(action2button,'String','Next');
    set(action2button,'Callback','Asufit(''absPlotUp'');');
    set(action2button,'Visible','On');
    set(action2button,'Foregroundcolor',[1 0 0]);
    set(action3button,'Callback','Asufit(''absChangePlotMode'');');
    set(action3button,'Visible','On');
    set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
          if (abstimelin == 0)
            tempt=t;  
            tempind= find(tempt>1);
            tempt(tempind)= 10.^(tempt(tempind)-1);
            set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
          else
             set(absCurrentTimeBox,'String',num2str(t(absInd2)));
          end
    set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
    if (isempty(fit))
        axes(absDataGrid1);
        absPlot1 = plot([t(absInd2) t(absInd2)],[min(min(data)) max(max(data))],'g',...
           t(absLeftEnd1:absRightEnd1),data(absInd1,absLeftEnd1:absRightEnd1),'b',...
           t(absLeftEnd1:absRightEnd1),zeros(size(t(absLeftEnd1:absRightEnd1))),'k','erasemode','xor');
        set(absPlot1,'LineWidth',2);
        axis([t(absLeftEnd1) t(absRightEnd1) min(min(data)) max(max(data))]);   
          if (abstimelin == 0)
            set(absDataGrid1, 'XTickMode', 'manual');
            tempticks = [ceil(min(t)):floor(max(t))];
            set(absDataGrid1, 'XTick', tempticks);
            set(absDataGrid1, 'XTickLabelMode', 'manual');
            tempind= find(tempticks>1);
            tempticks(tempind)= 10.^(tempticks(tempind)-1);
            set(absDataGrid1, 'XTickLabel',tempticks);
            set(absResidGrid1, 'XTickMode', 'manual');
            set(absResidGrid1, 'XTick', tempticks);
            set(absResidGrid1, 'XTickLabelMode', 'manual');
            set(absResidGrid1, 'XTickLabel',tempticks);
          else
            set(absDataGrid1, 'XTickMode', 'auto');
            set(absDataGrid1, 'XTickLabelMode', 'auto');
            set(absResidGrid1, 'XTickMode', 'auto');
            set(absResidGrid1, 'XTickLabelMode', 'auto');
          end

        axes(absDataGrid2);
        absPlot2 = plot([wavelength(absInd1) wavelength(absInd1)],[min(min(data)) max(max(data))],'g',wavelength(absLeftEnd2:absRightEnd2),data(absLeftEnd2:absRightEnd2,absInd2),'b',wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k','erasemode','xor');
        set(absPlot2,'LineWidth',2);
        axis([wavelength(absLeftEnd2) wavelength(absRightEnd2) min(min(data)) max(max(data))]);
        axes(absResidGrid1);
        absResidPlot1 = plot(t(absLeftEnd1:absRightEnd1),zeros(size(t(absLeftEnd1:absRightEnd1))),'k','erasemode','xor');
        axis([t(absLeftEnd1) t(absRightEnd1) -1 1]);
           if (abstimelin == 0)
            tempticks = [ceil(min(t)):floor(max(t))];
            set(absResidGrid1, 'XTickMode', 'manual');
            set(absResidGrid1, 'XTick', tempticks);
            tempind= find(tempticks>1);
            tempticks(tempind)= 10.^(tempticks(tempind)-1);
            set(absResidGrid1, 'XTickLabelMode', 'manual');
            set(absResidGrid1, 'XTickLabel',tempticks);
          else
            set(absResidGrid1, 'XTickMode', 'auto');
            set(absResidGrid1, 'XTickLabelMode', 'auto');
          end
        axes(absResidGrid2);
        absResidPlot2 = plot(wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k','erasemode','xor');
        axis([wavelength(absLeftEnd2) wavelength(absRightEnd2) -1 1]);
        set(absPlot1(1),'ButtonDownFcn','Asufit(''absPlot1Button'');');
        set(absPlot1(2),'ButtonDownFcn','Asufit(''absAxes1ButtonDown'');');
        set(absDataGrid1,'ButtonDownFcn','Asufit(''absAxes1ButtonDown'');');
        set(absPlot2(1),'ButtonDownFcn','Asufit(''absPlot2Button'');');
        set(absPlot2(2),'ButtonDownFcn','Asufit(''absAxes2ButtonDown'');');
        set(absDataGrid2,'ButtonDownFcn','Asufit(''absAxes2ButtonDown'');');
    else        
        axes(absDataGrid1)
        absPlot1 = plot([t(absInd2) t(absInd2)],[min(min(data)) max(max(data))],'g',t(absLeftEnd1:absRightEnd1),data(absInd1,absLeftEnd1:absRightEnd1),'b',t(absLeftEnd1:absRightEnd1),fit(absInd1 - absLeftEnd2 + 1,:),'r',t(absLeftEnd1:absRightEnd1),zeros(size(t(absLeftEnd1:absRightEnd1))),'k','erasemode','xor');
        set(absPlot1,'LineWidth',2);
        axis([t(absLeftEnd1) t(absRightEnd1) min([min(data) min(fit)]) max([max(data) max(fit)])]);
           if (abstimelin == 0)
            set(absDataGrid1, 'XTickMode', 'manual');
            tempticks = [ceil(min(t)):floor(max(t))];
            set(absDataGrid1, 'XTick', tempticks);
            set(absDataGrid1, 'XTickLabelMode', 'manual');
            tempind= find(tempticks>1);
            tempticks(tempind)= 10.^(tempticks(tempind)-1);
            set(absDataGrid1, 'XTickLabel',tempticks);
          else
            set(absDataGrid1, 'XTickMode', 'auto');
            set(absDataGrid1, 'XTickLabelMode', 'auto');
          end
        axes(absDataGrid2);
        absPlot2 = plot([wavelength(absInd1) wavelength(absInd1)],[min(min(data)) max(max(data))],'g',wavelength(absLeftEnd2:absRightEnd2),data(absLeftEnd2:absRightEnd2,absInd2),'b',wavelength(absLeftEnd2:absRightEnd2),fit(:,absInd2 - absLeftEnd1 + 1),'r',wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k','erasemode','xor');
        set(absPlot2,'LineWidth',2);
        axis([wavelength(absLeftEnd2) wavelength(absRightEnd2) min([min(data) min(fit)]) max([max(data) max(fit)])]);
        axes(absResidGrid1);
        absResidPlot1 = plot(t(absLeftEnd1:absRightEnd1),fitError(absInd1 - absLeftEnd2 + 1,:),'r',t(absLeftEnd1:absRightEnd1),zeros(size(t(absLeftEnd1:absRightEnd1))),'k',[t(absLeftEnd1) t(absLeftEnd1)],[min(min(fitError)) max(max(fitError))],'w','erasemode','xor');
        axis([t(absLeftEnd1) t(absRightEnd1) min(min(fitError)) max(max(fitError))]);
           if (abstimelin == 0)
            tempticks = [ceil(min(t)):floor(max(t))];
            set(absResidGrid1, 'XTickMode', 'manual');
            set(absResidGrid1, 'XTick', tempticks);
            tempind= find(tempticks>1);
            tempticks(tempind)= 10.^(tempticks(tempind)-1);
            set(absResidGrid1, 'XTickLabelMode', 'manual');
            set(absResidGrid1, 'XTickLabel',tempticks);
          else
            set(absResidGrid1, 'XTickMode', 'auto');
            set(absResidGrid1, 'XTickLabelMode', 'auto');
          end
        axes(absResidGrid2);
        absResidPlot2 = plot(wavelength(absLeftEnd2:absRightEnd2),fitError(:,absInd2 - absLeftEnd1 + 1),'r',wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k',[wavelength(absLeftEnd2) wavelength(absLeftEnd2)],[min(min(fitError)) max(max(fitError))],'w','erasemode','xor');
        axis([wavelength(absLeftEnd2) wavelength(absRightEnd2) min(min(fitError)) max(max(fitError))]);
        set(absTaoAnsText,'Visible','on');
        set(absTaoAnsBox,'Visible','on');        
        set(absTaoAnsBox,'String',{num2str([taoFixed tao]);num2str(tzero);num2str(pulseWidth);num2str(disperCoeffs)});
        set(absErrorText,'Visible','on');
        set(absErrorBox,'Visible','on');        
        set(absErrorBox,'String',num2str(chisq));
        set(absPlot1(1),'ButtonDownFcn','Asufit(''absPlot1Button'');');
        set(absPlot1(2),'ButtonDownFcn','Asufit(''absAxes1ButtonDown'');');
        set(absPlot1(3),'ButtonDownFcn','Asufit(''absAxes1ButtonDown'');');
        set(absDataGrid1,'ButtonDownFcn','Asufit(''absAxes1ButtonDown'');');
        set(absPlot2(1),'ButtonDownFcn','Asufit(''absPlot2Button'');');
        set(absPlot2(2),'ButtonDownFcn','Asufit(''absAxes2ButtonDown'');');
        set(absPlot2(3),'ButtonDownFcn','Asufit(''absAxes2ButtonDown'');');
        set(absDataGrid2,'ButtonDownFcn','Asufit(''absAxes2ButtonDown'');');
    end
    if (absPlotKinSpec == 1)
        str = 'Kin->Spec';
        str2 = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(absRightEnd2 - absLeftEnd2 + 1) '.'];
        set(textBox,'String',str2);
        if (absInd1 == absLeftEnd2)
            set(action1button,'Foregroundcolor',[.5 .5 .5]);
        end
        if (absInd1 == absRightEnd2)
            set(action2button,'Foregroundcolor',[.5 .5 .5]);
        end
    else
        str = 'Spec->Kin';
        str2 = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(absRightEnd1 - absLeftEnd1 + 1) '.'];
        set(textBox,'String',str2);
        if (absInd2 == absLeftEnd1)
            set(action1button,'Foregroundcolor',[.5 .5 .5]);
        end
        if (absInd2 == absRightEnd1)
            set(action2button,'Foregroundcolor',[.5 .5 .5]);
        end
    end    
    set(action3button,'String',str);
    set(action3button,'ForegroundColor',[1 0 0]);
        
elseif strcmp(action, 'absPlotUp')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the next SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (absPlotKinSpec == 1)
        if (absInd1 >= absRightEnd2) % No next plot
            absInd1 = absRightEnd2;
        else
            set(action2button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd1 = absInd1 + 1;
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            if (isempty(fit))
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);
            else
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot1(3),'YData',fit(absInd1 - absLeftEnd2 + 1,:));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);               
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                               
            end
            if (absInd1 == absRightEnd2) % Now no next plot either
                set(action2button,'Foregroundcolor',[.5 .5 .5]);
                set(action1button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0]);
            end
        end
        str = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(absRightEnd2 - absLeftEnd2 + 1) '.'];
        set(textBox,'String',str);
    else
        if (absInd2 >= absRightEnd1) % No next plot
            absInd2 = absRightEnd1;
        else
            set(action2button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd2 = absInd2 + 1;
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            if (isempty(fit))
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);
            else
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot2(3),'YData',fit(:,absInd2 - absLeftEnd1 + 1));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);               
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                   
            end
            if (absInd2== absRightEnd2) % Now no next plot either
                set(action2button,'Foregroundcolor',[.5 .5 .5]);
                set(action1button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0]);
            end
        end
        str = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(absRightEnd1 - absLeftEnd1 + 1) '.'];
        set(textBox,'String',str);
    end
    
elseif strcmp(action, 'absPlotDown')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the previous SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (absPlotKinSpec == 1)
        if (absInd1 >= absLeftEnd2 + 1)
            set(action1button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd1 = absInd1 - 1;        
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            if (isempty(fit))
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                    
            else
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot1(3),'YData',fit(absInd1 - absLeftEnd2 + 1,:));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);               
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));   
            end
            if (absInd1 == absLeftEnd2) % Now no next plot either
                set(action1button,'Foregroundcolor',[.5 .5 .5]);
                set(action2button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0])
            end  
        else
            absInd1 = absLeftEnd2;
        end
        str = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(absRightEnd2 - absLeftEnd2 + 1) '.'];
        set(textBox,'String',str);    
    else
        if (absInd2 >= absLeftEnd1 + 1)
            set(action1button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd2 = absInd2 - 1;        
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            if (isempty(fit))
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                    
            else
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot2(3),'YData',fit(:,absInd2 - absLeftEnd1 + 1));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]); 
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                                 
            end
            if (absInd2 == absLeftEnd1) % Now no next plot either
                set(action1button,'Foregroundcolor',[.5 .5 .5]);
                set(action2button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0])
            end  
        else
            absInd2 = absLeftEnd1;
        end
        str = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(absRightEnd1 - absLeftEnd1 + 1) '.'];
        set(textBox,'String',str);    
    end
elseif (strcmp(action, 'absChangePlotMode'))
    set(action3button,'ForegroundColor',[0 1 0]);
    drawnow
    if (absPlotKinSpec == 1)
        absPlotKinSpec = 2;
    else
        absPlotKinSpec = 1;
    end
    Asufit('absPlot');
elseif strcmp(action, 'absPlot1Move')    
        pt = get(absDataGrid1,'currentpoint');
        pt = find(t(absLeftEnd1:absRightEnd1) <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
%            if (pt < absInd1)
                absInd2 = pt + absLeftEnd1 - 1;
                set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
                if (abstimelin == 0)
                  tempt=t;  
                  tempind= find(tempt>1);
                  tempt(tempind)= 10.^(tempt(tempind)-1);
                  set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
                else
                  set(absCurrentTimeBox,'String',num2str(t(absInd2)));
                end
                set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));                
                if (isempty(fit))
                    set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                    set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                    
                else
                    set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                    set(absPlot2(3),'YData',fit(:,absInd2 - absLeftEnd1 + 1));
                    set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);               
                    set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                    set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                                           
                end
                str = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(absRightEnd1 - absLeftEnd1 + 1) '.'];
                set(textBox,'String',str);
                drawnow
%            end
        end
elseif strcmp(action, 'absPlot2Move')    
        pt = get(absDataGrid2,'currentpoint');
        pt = find(wavelength(absLeftEnd2:absRightEnd2) <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
%            if (pt < absInd1)
                absInd1 = pt + absLeftEnd2 - 1;
                set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
                if (abstimelin == 0)
                  tempt=t;  
                  tempind= find(tempt>1);
                  tempt(tempind)= 10.^(tempt(tempind)-1);
                  set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
                else
                  set(absCurrentTimeBox,'String',num2str(t(absInd2)));
                end
                set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
                str2 = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(absRightEnd2 - absLeftEnd2 + 1) '.'];
                set(textBox,'String',str2);
                if (isempty(fit))
                    set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                    set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                    
                else
                    set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                    set(absPlot1(3),'YData',fit(absInd1 - absLeftEnd2 + 1,:));
                    set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);               
                    set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                    set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                       
                end
                drawnow
%            end
        end
elseif strcmp(action, 'absPlot1Button')
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absPlotButtonUp'');');
    drawnow
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absPlot1Move'');');
elseif strcmp(action, 'absPlot2Button')    
    set(mainWindow,'WindowButtonUpFcn','Asufit(''absPlotButtonUp'');');
    drawnow
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''absPlot2Move'');');
elseif strcmp(action, 'absAxes1ButtonDown')
        set(mainWindow,'WindowButtonUpFcn','Asufit(''absPlotButtonUp'');');
        drawnow
        pt = get(absDataGrid1,'currentpoint');
        pt = find(t(absLeftEnd1:absRightEnd1) <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            absInd2 = pt + absLeftEnd1 - 1;
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            str2 = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(absRightEnd1 - absLeftEnd1 + 1) '.'];
            set(textBox,'String',str2);
            if (isempty(fit))
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                    
            else
                set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
                set(absPlot2(3),'YData',fit(:,absInd2 - absLeftEnd1 + 1));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);               
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                   
             end
             drawnow
             set(mainWindow,'WindowButtonMotionFcn','Asufit(''absPlot1Move'');');
        end
    
elseif strcmp(action, 'absAxes2ButtonDown')
        set(mainWindow,'WindowButtonUpFcn','Asufit(''absPlotButtonUp'');');
        drawnow
        pt = get(absDataGrid2,'currentpoint');
        pt = find(wavelength(absLeftEnd2:absRightEnd2) <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            absInd1 = pt + absLeftEnd2 - 1;
            set(absCurrentWlBox,'String',num2str(wavelength(absInd1)));
            if (abstimelin == 0)
              tempt=t;  
              tempind= find(tempt>1);
              tempt(tempind)= 10.^(tempt(tempind)-1);
              set(absCurrentTimeBox,'String',num2str(tempt(absInd2)));
            else
              set(absCurrentTimeBox,'String',num2str(t(absInd2)));
            end
            set(absAbsorbBox,'String',num2str(data(absInd1,absInd2)));
            str2 = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(absRightEnd2 - absLeftEnd2 + 1) '.'];
            set(textBox,'String',str2);
            if (isempty(fit))
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                    
             else
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot1(3),'YData',fit(absInd1 - absLeftEnd2 + 1,:));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);               
                set(absResidPlot1(1),'YData',fitError(absInd1 - absLeftEnd2 + 1,:));
                set(absResidPlot2(1),'YData',fitError(:,absInd2 - absLeftEnd1 + 1));                   
             end
             set(mainWindow,'WindowButtonMotionFcn','Asufit(''absPlot2Move'');');
             set(mainWindow,'WindowButtonUpFcn','Asufit(''absPlotButtonUp'');');
             drawnow
        end
    
elseif strcmp(action, 'absPlotButtonUp')    
    set(mainWindow,'WindowButtonMotionFcn',';');
    set(mainWindow,'WindowButtonUpFcn',';');
elseif (strcmp(action, 'dispersionCorrection'))
    %Dispersion correction using interp1 function
    % yy=phcip im matlab 6 gives nice interpolation
    numCurves = size(data,1);
    numTimes = size(data,2);
    dispersionVector = polyval(disperCoeffs,wavelength);
    lowerLimit = min(dispersionVector);
    dispersionVector = dispersionVector - lowerLimit;
    lowerLimit = 0;
    upperLimit = max(dispersionVector);
      if (upperLimit < 0)
          upperLimit = 0;
          msgbox('Dispersion correction values are not correct. Please input the correct values!');
      else
        if numTimePoints2>0
           dataTemp=data;
           for dispInd1 = 2:numCurves
              shift = dispersionVector(1,dispInd1);
              ttemp=t-shift;
              dataTemp(dispInd1,:) =interp1(ttemp,data(dispInd1,:),t,'spline');
           end
           top1=round(upperLimit/(t(2)-t(1)));
           top2=round(upperLimit/(t(length(t))-t(length(t)-1)));
           data=[dataTemp(:,1:numTimePoints1-top1) dataTemp(:,numTimePoints1+1:numTimePoints1+numTimePoints2-top2)];
           t=[t(1:numTimePoints1-top1) t(numTimePoints1+1:numTimePoints1+numTimePoints2-top2)];
           absLeftEnd1 = 1;
           absRightEnd1 = length(t);
        else
           dataTemp=data;
           for dispInd1 = 2:numCurves
              shift = dispersionVector(1,dispInd1);
              ttemp=t-shift;
              dataTemp(dispInd1,:) =interp1(ttemp,data(dispInd1,:),t,'spline');
           end
           top=ceil(upperLimit/(t(2)-t(1)));
           data=dataTemp(:,1:numTimePoints1-top);
           t=t(1:numTimePoints1-top);
           absLeftEnd1 = 1;
           absRightEnd1 = length(t);
        end
        dispersionStatus = 2;
        absInd1=1;
        absInd2=1;
        asufit('plot');
        msgbox('Dispersion correction completed successfully'); 
     end
elseif (strcmp(action, 'restoreData'))
      load datatemp;
        absYTicks(1)= dataMin;
        for i = 2:5
            absYTicks(i) = absYTicks(i-1) + (dataMax - dataMin)/4;
        end
        absYTicks = sort([absYTicks 0]);
        analysisType = 2;
        xTol = .01;
        fTol = .001;
        maxIt = 1000;
        absLeftEnd1 = 1;
        absRightEnd1 = length(t);
        absLeftEnd2 = 1;
        absRightEnd2 = length(wavelength);
        analysisType = 2;
        dispersionStatus = 0;
        svdAnal = 0;
        fit = [];
        Asufit('allOff')
        Asufit('abs')
        
elseif(strcmp(action, 'svdAnalysis'))    
    asufit('absPlotOff')
    asufit('absOptionsOff')
    set(svdbutton,'ForegroundColor',[0 1 0]); 
    set(optionsbutton,'ForegroundColor',[1 0 0]);
    set(plotbutton,'ForegroundColor',[1 0 0]);  
    if (isempty(svdAnal))
        svdAnal = 0;
    end
    if (svdAnal == 0)
        str = 'Performing SVD. . .';
        set(textBox,'String',str);
        drawnow;
        [U,S,V] = svd(data);
        numVecs = 5;
        set(textBox,'String','SVD complete');
        svdAnal = 1;
    end
    set(svdNumVecsBox,'String',num2str(numVecs));
    if (isempty(absPlotKinSpec))
        absPlotKinSpec = 1;
    end          
    if (isempty(absInd1))
        absInd1 = 1;             
    end
    if (isempty(absInd2))
        absInd2 = 1;         
    end
    set(svdNumVecsText,'visible','on');
    set(svdNumVecsBox,'visible','on');
    set(svdNumVecsBox,'String',num2str(numVecs));
    set(svdWeightText,'Visible','on');
    set(svdWeightBox,'Visible','on');
    str = sprintf('%d   %f\n',[(1:numVecs)' diag(S(1:numVecs,1:numVecs))]');
    set(svdWeightBox,'String',str);
    axes(svdGrid1);
    plot(t,V(:,1:numVecs),t,zeros(size(t)),'k');
    title('Kinetic components');
    axes(svdGrid2);
    plot(wavelength',U(:,1:numVecs),wavelength',zeros(length(wavelength)),'k');       
    title('Spectral components');
    axes(svdGrid3);
    semilogy(diag(S),'ro-');
    axis([1 25 1e-3 max(max(S))]);
    ylabel('Component weights');   
    
 elseif strcmp(action, 'svdPlotUp')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the next SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (absPlotKinSpec == 1)
        if (absInd1 >= length(wavelength)) % No next plot
            absInd1 = length(wavelength);
        else
            set(action2button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd1 = absInd1 + 1;
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));          
            set(absPlot1(2),'YData',data(absInd1,:));
            set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                              
            if (absInd1 == length(t)) % Now no next plot either
                set(action2button,'Foregroundcolor',[.5 .5 .5]);
                set(action1button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0]);
            end
        end
        str = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(length(wavelength)) '.'];
        set(textBox,'String',str);
    else
        if (absInd2 >= length(t)) % No next plot
            absInd2 = length(t);
        else
            set(action2button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd2 = absInd2 + 1;
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));
            set(absPlot2(2),'YData',data(:,absInd2));
            set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);
            if (absInd2== absRightEnd2) % Now no next plot either
                set(action2button,'Foregroundcolor',[.5 .5 .5]);
                set(action1button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0]);
            end
        end
        str = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(length(t)) '.'];
        set(textBox,'String',str);
    end
    
elseif strcmp(action, 'svdPlotDown')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument plots the previous SPC curve
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (absPlotKinSpec == 1)
        if (absInd1 >= 1)
            set(action1button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd1 = absInd1 - 1;        
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));        
            set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
            set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                                
            if (absInd1 == 1) % Now no next plot either
                set(action1button,'Foregroundcolor',[.5 .5 .5]);
                set(action2button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0])
            end  
        else
            absInd1 = 1;
        end
        str = ['Curve ' num2str(absInd1 - absLeftEnd2 + 1) ' of ' num2str(length(wavelength)) '.'];
        set(textBox,'String',str);    
    else
        if (absInd2 >= 1)
            set(action1button,'Foregroundcolor',[0 1 0]);
            drawnow
            absInd2 = absInd2 - 1;        
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));          
            set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
            set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                    
            if (absInd2 == 1) % Now no next plot either
                set(action1button,'Foregroundcolor',[.5 .5 .5]);
                set(action2button,'Foregroundcolor',[1 0 0]);        
            else
                set(action1button,'Foregroundcolor',[1 0 0]);
                set(action2button,'Foregroundcolor',[1 0 0])
            end  
        else
            absInd2 = 1;
        end
        str = ['Curve ' num2str(absInd2 - absLeftEnd1 + 1) ' of ' num2str(length(t)) '.'];
        set(textBox,'String',str);    
    end
elseif (strcmp(action, 'svdChangePlotMode'))
    set(action3button,'ForegroundColor',[0 1 0]);
    drawnow
    if (absPlotKinSpec == 1)
        absPlotKinSpec = 2;
    else
        absPlotKinSpec = 1;
    end
    Asufit('svdAnalysis');
elseif strcmp(action, 'svdPlot1Move')    
        pt = get(svdGrid1,'currentpoint');
        pt = find(t <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
%            if (pt < absInd1)
                absInd2 = pt;
                set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
                set(svdTimeBox,'String',num2str(t(absInd2)));
                set(absPlot2(2),'YData',data(:,absInd2));
                set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                                    
                 drawnow
%            end
        end
 elseif strcmp(action, 'svdPlot2Move')    
        pt = get(svdGrid2,'currentpoint');
        pt = find(wavelength <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
%            if (pt < absInd1)
                absInd1 = pt;
                set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
                set(svdTimeBox,'String',num2str(t(absInd2)));
                set(absPlot1(2),'YData',data(absInd1,absLeftEnd1:absRightEnd1));
                set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                               
                 drawnow
%            end
        end

elseif strcmp(action, 'svdPlot1Button')
    set(mainWindow,'WindowButtonUpFcn','Asufit(''svdPlotButtonUp'');');
    drawnow
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''svdPlot1Move'');');
elseif strcmp(action, 'svdPlot2Button')    
    set(mainWindow,'WindowButtonUpFcn','Asufit(''svdPlotButtonUp'');');
    drawnow
    set(mainWindow,'WindowButtonMotionFcn','Asufit(''svdPlot2Move'');');
elseif strcmp(action, 'svdAxes1ButtonDown')
        set(mainWindow,'WindowButtonUpFcn','Asufit(''svdPlotButtonUp'');');
        drawnow
        pt = get(svdGrid1,'currentpoint');
        pt = find(t <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            absInd2 = pt;
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));
            set(absPlot2(2),'YData',data(absLeftEnd2:absRightEnd2,absInd2));
            set(absPlot1(1),'XData',[t(absInd2) t(absInd2)]);                                             
            drawnow
            set(mainWindow,'WindowButtonMotionFcn','Asufit(''svdPlot1Move'');');
        end
    
elseif strcmp(action, 'svdAxes2ButtonDown')
        set(mainWindow,'WindowButtonUpFcn','Asufit(''svdPlotButtonUp'');');
        drawnow
        pt = get(svdGrid2,'currentpoint');
        pt = find(wavelength <= pt(1,1));
        if (pt)
            pt = pt(length(pt));
            absInd1 = pt;
            set(svdWavelengthBox,'String',num2str(wavelength(absInd1)));
            set(svdTimeBox,'String',num2str(t(absInd2)));
            set(absPlot1(2),'YData',data(absInd1,:));
            set(absPlot2(1),'XData',[wavelength(absInd1) wavelength(absInd1)]);                                            
            set(mainWindow,'WindowButtonMotionFcn','Asufit(''svdPlot2Move'');');
            set(mainWindow,'WindowButtonUpFcn','Asufit(''svdPlotButtonUp'');');
             drawnow
        end
elseif strcmp(action, 'svdPlotButtonUp')    
    set(mainWindow,'WindowButtonMotionFcn',';');
    set(mainWindow,'WindowButtonUpFcn',';');           
elseif (strcmp(action, 'svdGetNumVecs'))
    temp = str2num(get(svdNumVecsBox,'String'));
    if (temp)
        if (temp <= size(S,1) & temp > 0)
            numVecs = temp;
        end
    end
    Asufit('svdAnalysis');
elseif strcmp(action, 'svdGetWavelength')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets the current wavelength to the wavelength closest
% to user input
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    newWl = str2num(get(svdWavelengthBox,'String'));
    tempWl = abs(wavelength - newWl);
    temp = find(tempWl == min(tempWl));
    if (temp)
        absInd1 = temp;
        Asufit('svdAnalysis');
    end

elseif strcmp(action, 'svdGetTime')
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% This action argument sets the current time to the time closest to user input
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    newT = str2num(get(svdTimeBox,'String'));
    tempT = abs(t - newT);
    temp = find(tempT == min(tempT));
    if (temp)
        absInd2 = temp;
        Asufit('svdAnalysis');
    end

elseif (strcmp(action, 'showGlobalNew'))
    figure(svdSurfNew);
elseif (strcmp(action, 'showGlobalOld'))
    figure(svdSurfOld);
elseif (strcmp(action, 'closeGlobalViews'))
    close(svdSurfNew);
    close(svdSurfOld);   
    figure(mainWindow);
    set(svdGlobalTogHypertext,'ForegroundColor',[1 0 0]);
elseif (strcmp(action, 'ampSpecPlot'))
    ampPlotAll = 1;
    if (isempty(ampInd))
        ampInd = 1;
    end
    ampSpecWindow = ... % Window for viewing amplitude spectra
        figure(...
            'Name',        'Amplitude Spectra', ...
            'NumberTitle', 'off', ...
            'MenuBar', 'none',...
            'Units',       'normalized',...
            'Position',    [0.1 0.1 0.8 0.8],...
            'Color', [1 1 1]...
    );

    ampGrid = ... % Axes for the amplitude spectra plots
        axes(...
            'Position', [0.1 .3 0.76 0.48],...
            'DrawMode', 'Fast'...
    );
    ampUpbutton = ... % Button for "up" plot control
        uicontrol(gcf,...
            'Style',           'pushbutton',...
            'Units',           'normalized',...
            'Position',        [.9 .71 .08 .05],... %tells where to put it
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'String',          'Next',...
            'Callback',   'Asufit(''ampSpecPlotUp'');'...
    );
    ampDownbutton = ... % Button for "down" plot control
        uicontrol(gcf,...
            'Style',           'pushbutton',...
            'Units',           'normalized',...
            'Position',        [.9 .64 .08 .05],...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'String',          'Previous',...
            'Callback',   'Asufit(''ampSpecPlotDown'');'...
    );
    ampClosebutton = ... % Button to close the plot window
        uicontrol(gcf,...
            'Style',           'pushbutton',...
            'Units',           'normalized',...
            'Position',        [.9 .56 0.08 .05],...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'String',          'Close',...
            'Interruptible',   'On',...
            'Callback',   'Asufit(''ampSpecQuit'');'...
    );
    ampTextBox = ... % Text box to communicate with user
        uicontrol(gcf,...
            'Style',               'text',...
            'Units',               'normalized',...
            'Max',                 10,...
            'HorizontalAlignment', 'center',...
            'BackgroundColor',     [1 1 1],...
            'ForegroundColor',     [0 0 0],...
            'Position',            [0 .9 1 .05]...
    );
    ampSwitchbutton = ... % Hypertext to toggle betwene plot styles
        uicontrol(gcf,...
            'String',          'All->Single',...
            'Style',           'pushbutton',...
            'Units',           'normalized',...
            'Position',        [.9 .44 .08 .05],...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'Callback',   'Asufit(''ampSpecPlotSwitch'');'...
            );
     ampPrintbutton = ... % Hypertext to toggle betwene plot styles
        uicontrol(gcf,...
            'String',          'Print',...
            'Style',           'pushbutton',...
            'Units',           'normalized',...
            'Position',        [.9 .34 .08 .05],...
            'BackgroundColor', [0.8 0.8 0.8],...
            'ForegroundColor', [1 0 0],...
            'Callback',   'print;'...
    );

    uicontrol(gcf,...
                'Style',               'text', ...
                'Units',               'normalized', ...
                'String',              'Lifetimes',...
                'HorizontalAlignment', 'center',...
                'BackgroundColor',     [1 1 1],...
                'ForegroundColor',     [0 0 0],...
                'Position',            [0.1 .05 .1 .05]...
        );

    colorTemp = get(ampGrid,'colororder');
    for i = 1:length(taoAns)
        % Temporary uicontrols to show values of tao
            uicontrol(gcf,...
                'Style',               'text', ...
                'Units',               'normalized', ...
                'Max',                 10, ...
                'String',              num2str(taoAns(i)),...
                'HorizontalAlignment', 'center',...
                'BackgroundColor',     [1 1 1],...
                'ForegroundColor',     colorTemp(rem(i,6),:),...
                'Position',            [0.1+.1*i .05 .1 .05]...
        );
     end
  Asufit('ampSpecShow');

elseif(strcmp(action, 'ampSpecShow'))
   if (ampPlotAll == 0)
        plot(wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k',wavelength(absLeftEnd2:absRightEnd2),amps(:,ampInd));
        colorTemp = get(ampGrid,'colororder');
        temp = get(ampGrid,'children');
        set(temp,'color',colorTemp(rem(ampInd,6),:));
        str = ['Amplitude spectrum for tao number ' num2str(ampInd) ', where tao = ' num2str(taoAns(ampInd)) '.'];
        set(ampTextBox,'String',str);        
    else
        plot(wavelength(absLeftEnd2:absRightEnd2),amps,wavelength(absLeftEnd2:absRightEnd2),zeros(size(wavelength(absLeftEnd2:absRightEnd2))),'k');
        str = ['Decay Associated Spectra of the file ' name ' fitted with ' num2str(numTao) ' lifetimes.'];
        set(ampTextBox,'string',str);
     end
     xlabel('Wavelength, nm');
     ylabel('Delta Absorbance');
elseif(strcmp(action, 'ampSpecPlotUp'))
    if (ampInd < size(amps,2))
        ampInd = ampInd + 1;
        Asufit('ampSpecShow');
    end
elseif(strcmp(action, 'ampSpecPlotDown'))
    if (ampInd > 1)
        ampInd = ampInd - 1;
        Asufit('ampSpecShow');
    end
elseif(strcmp(action, 'ampSpecQuit'))
delete(ampSpecWindow);
%This is a hack fix
%Turn stuff off here
elseif(strcmp(action, 'ampSpecPlotSwitch'))
    if (ampPlotAll == 1)
%        set(amp_Upbutn,''visible'',''off'')
%        set(amp_Downbutn,''visible'',''off'')
        str = 'Single->All';
        ampPlotAll = 0;
    else
%         'set(amp_Upbutn,''visible'',''on'');,',...
%         'set(amp_Downbutn,''visible'',''on'');,',...
        str = 'All->Single';        
        ampPlotAll = 1;
    end
      set(ampSwitchbutton,'String',str);
      Asufit('ampSpecShow');
      
elseif (strcmp(action, 'abs3dPlot'))
     abs3dFig = ...
        figure( ...
            'Name','3D Absorbance Plot', ...
            'NumberTitle','off', ...
            'MenuBar', 'none',...
            'Units','normalized',...
            'Position',[0.15 .15 .7 .7],...
            'Visible','on'...
    );
    abs3dAxes = ...
        axes( ...
            'Position',[0.1 .1 0.75 0.75],...
            'Visible','off'...
    );

   surf(t,wavelength,data); 
   shading interp;
   colorbar;
   ylabel('Wavelength, nm');
   xlabel('Delay, ps');
   zlabel('Delta Absorbance');
   rotate3d on;
   ylim([min(wavelength) max(wavelength)]);
   xlim([min(t) max(t)]);
   zlim([min(min(data)) max(max(data))]);
   
elseif (strcmp(action, 'abstimechange'))
   if (abstimelin==1)
      temp = find(t>1);
      t(temp)= log10(t(temp));
      t(temp)=t(temp)+1;
      abstimelin=0;
      asufit('plot');
   else
      temp = find(t>1);
      t(temp)= 10.^(t(temp)-1);
      abstimelin=1;
      asufit('plot');
   end
   
elseif (strcmp(action, 'printAbs'))
   print
   
elseif (strcmp(action, 'zoomon'))
   zoom on;   
   set(mainWindow, 'Pointer', 'crosshair');

elseif (strcmp(action, 'zoomoff'))
   set(mainWindow, 'Pointer', 'arrow');
   zoom out;
   zoom off;
   
elseif (strcmp(action, 'gridson'))
   set(spcDataGrid, 'XGrid', 'on', 'YGrid', 'on');
   set(spcResidGrid, 'XGrid', 'on', 'YGrid', 'on');
   set(absDataGrid1, 'XGrid', 'on', 'YGrid', 'on');
   set(absDataGrid2, 'XGrid', 'on', 'YGrid', 'on');
   set(absResidGrid1, 'XGrid', 'on', 'YGrid', 'on');
   set(absResidGrid2, 'XGrid', 'on', 'YGrid', 'on');
   set(svdGrid1, 'XGrid', 'on', 'YGrid', 'on');
   set(svdGrid2, 'XGrid', 'on', 'YGrid', 'on');
   set(svdGrid3, 'XGrid', 'on', 'YGrid', 'on');
   set(absIntSetAxes1, 'XGrid', 'on', 'YGrid', 'on');
   set(absIntSetAxes2, 'XGrid', 'on', 'YGrid', 'on');
   set(spcIntSetAxes, 'XGrid', 'on', 'YGrid', 'on');
elseif (strcmp(action, 'gridsoff'))
   set(spcDataGrid, 'XGrid', 'off', 'YGrid', 'off');
   set(spcResidGrid, 'XGrid', 'off', 'YGrid', 'off');
   set(absDataGrid1, 'XGrid', 'off', 'YGrid', 'off');
   set(absDataGrid2, 'XGrid', 'off', 'YGrid', 'off');
   set(absResidGrid1, 'XGrid', 'off', 'YGrid', 'off');
   set(absResidGrid2, 'XGrid', 'off', 'YGrid', 'off');
   set(svdGrid1, 'XGrid', 'off', 'YGrid', 'off');
   set(svdGrid2, 'XGrid', 'off', 'YGrid', 'off');
   set(svdGrid3, 'XGrid', 'off', 'YGrid', 'off');
   set(absIntSetAxes1, 'XGrid', 'off', 'YGrid', 'off');
   set(absIntSetAxes2, 'XGrid', 'off', 'YGrid', 'off');
   set(spcIntSetAxes, 'XGrid', 'off', 'YGrid', 'off');
elseif (strcmp(action, 'selectfitmodel'))
    %temp = {'Simple sum of exponentials';'Sequential model A->B->...'}
    [fitmodel,ok] = listdlg('PromptString','Select a model for fitting:',...
                'SelectionMode','single',...
                'ListString',{'Simple sum of exponentials';'Sequential model A->B->...'});
            if (ok == 0)
                fitmodel=1;
            end
end


